package com.tourism.webadmin.app.website.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.validator.UpdateGroup;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

import java.util.Date;

/**
 * 包车司机Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车司机Dto对象")
@Data
public class TourCarDriverToWebDto {

    /**
     * id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 司机名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "司机名称。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 性别。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "性别。可支持等于操作符的列表数据过滤。")
    private Integer sex;

    /**
     * 年龄。
     */
    @Schema(description = "年龄。")
    private String age;

    /**
     * 查询开始年龄
     */
    private String ageBegin;
    /**
     * 查询开始年龄
     */
    private String ageEnd;

    /**
     * 驾龄。
     */
    @Schema(description = "驾龄。")
    private Integer drivingYears;

    /**
     * 司机照片。
     */
    @Schema(description = "司机照片。")
    private String image;

    /**
     * 评分。
     */
    @Schema(description = "评分。")
    private Integer mark;

    /**
     * 所属区域。
     */
    @TableField(value = "belong_tab")
    private Long belongTab;

    /**
     * 年龄。
     */
    @Schema(description = "年龄。")
    private Integer type;

    /**
     * name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 分页号码，从1开始计数。
     */
    @Schema(description = "分页号码。")
    private Integer pageNum = 1;

    /**
     * 每页大小。
     */
    @Schema(description = "每页大小。")
    private Integer pageSize = 12;

    /**
     * 是否统计totalCount。
     */
    @Schema(description = "是否统计totalCount。")
    private Boolean count = true;

    /**
     * 排序。
     */
    @Schema(description = "排序。")
    private MyOrderParam orderParam;

    /**
     * 出发日期
     */
    private String startDate;

    /**
     * 出发日期
     */
    private String endDate;

    /**
     * 包车天数
     */
    private Integer days;
}
