package com.tourism.webadmin.app.website.dto;

import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.webadmin.back.model.constant.Enable;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

/**
 * 包车车型Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车车型Dto对象")
@Data
public class TourCarModelToWebDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 车型名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车型名称。可支持等于操作符的列表数据过滤。")
    private String typeName;

    /**
     * 车型图片。
     */
    @Schema(description = "车型图片。")
    private String typeUrl;

    /**
     * 是否启用。
     */
    @Schema(description = "0启用，1禁用。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用为无效值！")
    private Integer enable;

    /**
     * 排序。
     */
    @Schema(description = "排序。")
    private Integer showOrder;

    /**
     * 删除标记(1: 正常 -1: 已删除)。
     */
    @Schema(description = "删除标记(1: 正常 -1: 已删除)。")
    private Integer dataState;

    /**
     * type_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 分页号码，从1开始计数。
     */
    @Schema(description = "分页号码。")
    private Integer pageNum = 0;

    /**
     * 每页大小。
     */
    @Schema(description = "每页大小。")
    private Integer pageSize = 100;

    /**
     * 是否统计totalCount。
     */
    @Schema(description = "是否统计totalCount。")
    private Boolean count = true;
}
