package com.tourism.webadmin.app.website.dto;

import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.webadmin.back.model.constant.TourCarOrderState;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

import java.util.Date;

/**
 * 包车订单Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车订单Dto对象")
@Data
public class TourCarOrderToWebDto {

    /**
     * id。
     */
    @Schema(description = "id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 订单号。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "订单号。可支持等于操作符的列表数据过滤。")
    private String orderNo;

    /**
     * 包车车型id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "包车车型id。可支持等于操作符的列表数据过滤。")
    private Long categoryId;

    /**
     * 司机id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "司机id。可支持等于操作符的列表数据过滤。")
    private Long driverId;

    /**
     * 开始时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "开始时间。可支持等于操作符的列表数据过滤。")
    private Date startTime;

    /**
     * 结束时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "结束时间。可支持等于操作符的列表数据过滤。")
    private Date endTime;

    /**
     * 包车天数。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "包车天数。可支持等于操作符的列表数据过滤。")
    private Integer days;

    /**
     * 起点。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "起点。可支持等于操作符的列表数据过滤。")
    private Long startPlaceId;

    /**
     * 联系电话。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系电话。可支持等于操作符的列表数据过滤。")
    private String connectPhone;

    /**
     * 联系姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系姓名。可支持等于操作符的列表数据过滤。")
    private String connectName;

    /**
     * 联系人性别。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系人性别。可支持等于操作符的列表数据过滤。")
    private Integer connectSex;

    /**
     * 备注。
     */
    @Schema(description = "备注。")
    private String remark;

    /**
     * 订单状态(1.已下单;2.已取消;3. 已完成)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "订单状态(1.已下单;2.已取消;3. 已完成)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = TourCarOrderState.class, message = "数据验证失败，订单状态(1.已下单;2.已取消;3. 已完成)为无效值！")
    private Integer state;

    /**
     * 车型评分。
     */
    @Schema(description = "车型评分。")
    private Integer categoryMark;

    /**
     * 司机评分。
     */
    @Schema(description = "司机评分。")
    private Integer driveMark;

    /**
     * 组织id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "组织id。可支持等于操作符的列表数据过滤。")
    private Long deptId;

    @Schema(description = "订单是否已打分")
    private Integer marking;

    /**
     * startTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "startTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String startTimeStart;

    /**
     * startTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "startTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String startTimeEnd;

    /**
     * endTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String endTimeStart;

    /**
     * endTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String endTimeEnd;

    /**
     * connect_phone / connect_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    /**
     * 分页号码，从1开始计数。
     */
    @Schema(description = "分页号码。")
    private Integer pageNum = 1;

    /**
     * 每页大小。
     */
    @Schema(description = "每页大小。")
    private Integer pageSize = 10;

    /**
     * 是否统计totalCount。
     */
    @Schema(description = "是否统计totalCount。")
    private Boolean count = true;

    /**
     * 排序。
     */
    @Schema(description = "排序。")
    private MyOrderParam orderParam;
}
