package com.tourism.webadmin.app.website.dto;

import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.validator.UpdateGroup;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

import java.math.BigDecimal;

/**
 * 拼团设置从表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "拼团设置从表Dto对象")
@Data
public class TourProjectGroupPurchaseDetailDtoToWebDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 关联 拼团设置的code。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "关联 拼团设置的code。可支持等于操作符的列表数据过滤。")
    private Long groupPurchaseCode;

    /**
     * 成人价格。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "成人价格。可支持等于操作符的列表数据过滤。")
    private BigDecimal adultPrice;

    /**
     * 儿童价格。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "儿童价格。可支持等于操作符的列表数据过滤。")
    private BigDecimal childrenPrice;

    /**
     * 阶梯的人数(下限)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "阶梯的人数(下限)。可支持等于操作符的列表数据过滤。")
    private Integer minCount;

    /**
     * 阶梯的人数(上限)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "阶梯的人数(上限)。可支持等于操作符的列表数据过滤。")
    private Integer maxCount;

    /**
     * adultPrice 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "adultPrice 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal adultPriceStart;

    /**
     * adultPrice 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "adultPrice 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal adultPriceEnd;

    /**
     * childrenPrice 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "childrenPrice 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal childrenPriceStart;

    /**
     * childrenPrice 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "childrenPrice 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal childrenPriceEnd;

    /**
     * minCount 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "minCount 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer minCountStart;

    /**
     * minCount 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "minCount 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer minCountEnd;

    /**
     * maxCount 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "maxCount 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer maxCountStart;

    /**
     * maxCount 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "maxCount 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer maxCountEnd;

    /**
     * 分页号码，从1开始计数。
     */
    @Schema(description = "分页号码。")
    private Integer pageNum = 1;

    /**
     * 每页大小。
     */
    @Schema(description = "每页大小。")
    private Integer pageSize = 10;

    /**
     * 是否统计totalCount。
     */
    @Schema(description = "是否统计totalCount。")
    private Boolean count = true;

    /**
     * 排序。
     */
    @Schema(description = "排序。")
    private MyOrderParam orderParamList;
}
