package com.tourism.webadmin.app.website.dto;

import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.webadmin.back.model.constant.InterviewFlag;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

@Schema(description = "签证信息Dto对象")
@Data
public class TourVisaSearchDto extends PageDto{

    /**
     * 签证id。
     */
    @Schema(description = "签证id。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String id;

    /**
     * 图片地址。
     */
    @Schema(description = "图片地址。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String img;

    /**
     * 名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String visaTitle;

    /**
     * 类型id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "类型id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String visaTypeId;

    /**
     * 国家编码。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "国家编码。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String countryCodeId;

    /**
     * 使馆名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "使馆名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String embassyName;

    /**
     * 使馆地址。
     */
    @Schema(description = "使馆地址。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String embassyAddress;

    /**
     * 是否面签 0 是 1否。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否面签 0 是 1否。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @ConstDictRef(constDictClass = InterviewFlag.class, message = "数据验证失败，是否面签 0 是 1否为无效值！")
    private Integer interviewFlag;

    /**
     * 有效期（日）。
     */
    @Schema(description = "有效期（日）。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer validity;

    /**
     * 停留天数。
     */
    @Schema(description = "停留天数。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer idleDays;

    /**
     * 预计办理天数。
     */
    @Schema(description = "预计办理天数。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer processingDays;

    /**
     * 预计价格。
     */
    @Schema(description = "预计价格。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer floorPrice;

    /**
     * 受理范围。
     */
    @Schema(description = "受理范围。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String acceptanceRange;

    /**
     * 入境次数。
     */
    @Schema(description = "入境次数。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer entriesNum;

    /**
     * 预定须知。
     */
    @Schema(description = "预定须知。", requiredMode = Schema.RequiredMode.REQUIRED)
    private String dataText;

    /**
     * 出签地编码。
     */
    @Schema(description = "出签地编码。", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long issueVisaCode;

    /**
     * visa_title / embassy_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    @Schema(description = "价格排序")
    private Integer PriceSortFlag;
}
