package com.tourism.webadmin.app.wechat.vo.tourism;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.tourism.webadmin.back.model.TourismFile;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;

/**
 * IndexDestinationProjectVo对象。
 *
 * @author huangwen
 * @date 2024/10/29
 */
@Schema(description = "IndexDestinationProjectVo")
@Data
public class IndexDestinationProjectVo {

    /**
     * 主键id。
     */
    @Schema(description = "主键id")
    private String id;

    /**
     * 项目标题。
     */
    @Schema(description = "项目标题")
    private String projectTitle;

    /**
     * 项目简述。
     */
    @Schema(description = "项目简述")
    private String remarks;

    /**
     * 是否热点。
     */
    @Schema(description = "是否热点，0非热点，1热点")
    private Integer isHotspot;

    /**
     * 标签。
     */
    @Schema(description = "标签")
    private String projectLabel;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序")
    private Integer showOrder;

    /**
     * 是否启用。
     */
    @Schema(description = "是否启用，0禁用，1启用")
    private Integer enable;

    /**
     * 所属分类。
     */
    @Schema(description = "所属分类")
    private String belongTab;

    /**
     * 出发地。
     */
    @Schema(description = "出发地")
    private String startPlace;

    /**
     * 目的地。
     */
    @Schema(description = "目的地")
    private String endPlace;

    /**
     * 总天数。
     */
    @Schema(description = "总天数")
    private String countTimes;

    /**
     * 项目价格。
     */
    @Schema(description = "项目价格")
    private BigDecimal price;

    /**
     * 项目展示图片。
     */
    @Schema(description = "项目展示图片")
    private String tourismUrl;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位")
    private String priceUnit;

    /**
     * 服务保障。
     */
    @Schema(description = "服务保障")
    private String serviceEnsure;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话")
    private String contactNumber;

    /**
     * 产品卖点。
     */
    @Schema(description = "产品卖点")
    private String sellingPoint;

    /**
     * 短标题。
     */
    @Schema(description = "短标题")
    private String shortTitle;

    /**
     * 短描述。
     */
    @Schema(description = "短描述")
    private String shortDescription;

    /**
     * 首页热门图片。
     */
    @Schema(description = "首页热门图片")
    private String homeHotPicture;

    /**
     * 联系人描述。
     */
    @Schema(description = "联系人描述")
    private String contactDescription;

    /**
     * 游记banner。
     */
    @Schema(description = "游记banner")
    private String travelNotesBanner;

    /**
     * 点赞量。
     */
    @Schema(description = "点赞量")
    private Integer likeCount;

    /**
     * 浏览量。
     */
    @Schema(description = "浏览量")
    private Integer pageViewCount;

    /**
     * 人物关系。
     */
    @Schema(description = "人物关系")
    private String role;

    /**
     * 出发时间。
     */
    @Schema(description = "出发时间")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private LocalDate departureTime;

    /**
     * 人均费用。
     */
    @Schema(description = "人均费用")
    private String averageCost;

    /**
     * 推荐指数。
     */
    @Schema(description = "推荐指数")
    private Integer recommendationRate;

//    /**
//     * 上传的联系二维码。
//     */
//    @Schema(description = "上传的联系二维码")
//    private String contactCode;

//    /**
//     * 是否优质推荐。
//     */
//    @Schema(description = "是否优质推荐")
//    private Integer isQualityRecommendation;


    /**
     * id 的一对一关联数据对象，数据对应类型为TourismFileVo。
     */
    @Schema(description = "id 的一对一关联数据对象，数据对应类型为TourismFileVo")
    private Map<String, Object> tourismFile;

    /**
     * id 的一对一关联数据对象，数据对应类型为TourismContentVo。
     */
    @Schema(description = "id 的一对一关联数据对象，数据对应类型为TourismContentVo")
    private Map<String, Object> tourismContent;

    /**
     * belongTab 字典关联数据。
     */
    @Schema(description = "belongTab 字典关联数据")
    private Map<String, Object> belongTabDictMap;

    /**
     * isHotspot 常量字典关联数据。
     */
    @Schema(description = "isHotspot 常量字典关联数据")
    private Map<String, Object> isHotspotDictMap;

    /**
     * enable 常量字典关联数据。
     */
    @Schema(description = "enable 常量字典关联数据")
    private Map<String, Object> enableDictMap;

    /**
     * tourismUrl 转换后的图片对象。
     */
    @Schema(description = "tourismUrl转换后的图片对象")
    private List<String> tourismUrlsAfterConvert;

    /**
     * homeHotPicture 转换后的图片对象。
     */
    @Schema(description = "homeHotPicture转换后的图片对象")
    private List<String> homeHotPicturesAfterConvert;

    /**
     * travelNotesBanner 转换后的图片对象。
     */
    @Schema(description = "travelNotesBanner转换后的图片对象")
    private List<String> travelNotesBannerAfterConvert;

//    /**
//     * contactCode 上传的联系二维码。
//     */
//    @Schema(description = "上传的联系二维码 contactCode转换后的图片对象")
//    private String contactCodeConvert;


    /**
     * fileList 项目图集。
     */
    @Schema(description = "fileList 项目图集")
    private List<TourismFile> fileList;
}
