package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 拼团返利记录操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "拼团返利记录管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourProjectGroupPurchaseRebate")
public class TourProjectGroupPurchaseRebateController {

    @Autowired
    private TourProjectGroupPurchaseRebateService tourProjectGroupPurchaseRebateService;

    /**
     * 新增拼团返利记录数据。
     *
     * @param tourProjectGroupPurchaseRebateDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourProjectGroupPurchaseRebateDto.id",
            "tourProjectGroupPurchaseRebateDto.priceStart",
            "tourProjectGroupPurchaseRebateDto.priceEnd"})
    @SaCheckPermission("tourProjectGroupPurchaseRebate.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourProjectGroupPurchaseRebateDto tourProjectGroupPurchaseRebateDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourProjectGroupPurchaseRebateDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate = MyModelUtil.copyTo(tourProjectGroupPurchaseRebateDto, TourProjectGroupPurchaseRebate.class);
        tourProjectGroupPurchaseRebate = tourProjectGroupPurchaseRebateService.saveNew(tourProjectGroupPurchaseRebate);
        return ResponseResult.success(tourProjectGroupPurchaseRebate.getId());
    }

    /**
     * 更新拼团返利记录数据。
     *
     * @param tourProjectGroupPurchaseRebateDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourProjectGroupPurchaseRebateDto.priceStart",
            "tourProjectGroupPurchaseRebateDto.priceEnd"})
    @SaCheckPermission("tourProjectGroupPurchaseRebate.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourProjectGroupPurchaseRebateDto tourProjectGroupPurchaseRebateDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourProjectGroupPurchaseRebateDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate = MyModelUtil.copyTo(tourProjectGroupPurchaseRebateDto, TourProjectGroupPurchaseRebate.class);
        TourProjectGroupPurchaseRebate originalTourProjectGroupPurchaseRebate = tourProjectGroupPurchaseRebateService.getById(tourProjectGroupPurchaseRebate.getId());
        if (originalTourProjectGroupPurchaseRebate == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourProjectGroupPurchaseRebateService.update(tourProjectGroupPurchaseRebate, originalTourProjectGroupPurchaseRebate)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除拼团返利记录数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourProjectGroupPurchaseRebate.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除拼团返利记录数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourProjectGroupPurchaseRebate.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的拼团返利记录列表。
     *
     * @param tourProjectGroupPurchaseRebateDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourProjectGroupPurchaseRebate.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourProjectGroupPurchaseRebateVo>> list(
            @MyRequestBody TourProjectGroupPurchaseRebateDto tourProjectGroupPurchaseRebateDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebateFilter = MyModelUtil.copyTo(tourProjectGroupPurchaseRebateDtoFilter, TourProjectGroupPurchaseRebate.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourProjectGroupPurchaseRebate.class);
        List<TourProjectGroupPurchaseRebate> tourProjectGroupPurchaseRebateList =
                tourProjectGroupPurchaseRebateService.getTourProjectGroupPurchaseRebateListWithRelation(tourProjectGroupPurchaseRebateFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourProjectGroupPurchaseRebateList, TourProjectGroupPurchaseRebateVo.class));
    }

    /**
     * 查看指定拼团返利记录对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourProjectGroupPurchaseRebate.view")
    @GetMapping("/view")
    public ResponseResult<TourProjectGroupPurchaseRebateVo> view(@RequestParam Long id) {
        TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate = tourProjectGroupPurchaseRebateService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourProjectGroupPurchaseRebate == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourProjectGroupPurchaseRebateVo tourProjectGroupPurchaseRebateVo = MyModelUtil.copyTo(tourProjectGroupPurchaseRebate, TourProjectGroupPurchaseRebateVo.class);
        return ResponseResult.success(tourProjectGroupPurchaseRebateVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourProjectGroupPurchaseRebate originalTourProjectGroupPurchaseRebate = tourProjectGroupPurchaseRebateService.getById(id);
        if (originalTourProjectGroupPurchaseRebate == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourProjectGroupPurchaseRebateService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
