package com.tourism.webadmin.back.dao;

import com.tourism.common.core.annotation.EnableDataPerm;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.webadmin.back.model.TourCarOrder;
import com.tourism.webadmin.back.model.TourCarCategory;
import com.tourism.webadmin.back.model.TourCarDriver;
import org.apache.ibatis.annotations.Param;

import java.util.*;

/**
 * 包车订单数据操作访问接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@EnableDataPerm
public interface TourCarOrderMapper extends BaseDaoMapper<TourCarOrder> {

    /**
     * 批量插入对象列表。
     *
     * @param tourCarOrderList 新增对象列表。
     */
    void insertList(List<TourCarOrder> tourCarOrderList);

    /**
     * 获取过滤后的对象列表。
     *
     * @param tourCarOrderFilter 主表过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 对象列表。
     */
    List<TourCarOrder> getTourCarOrderList(
            @Param("tourCarOrderFilter") TourCarOrder tourCarOrderFilter, @Param("orderBy") String orderBy);

    /**
     * 获取过滤后的对象列表。同时支持基于一对一从表字段的过滤条件。
     *
     * @param tourCarOrderFilter 主表过滤对象。
     * @param tourCarCategoryFilter 一对一从表过滤对象。
     * @param tourCarDriverFilter 一对一从表过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 对象列表。
     */
    List<TourCarOrder> getTourCarOrderListEx(
            @Param("tourCarOrderFilter") TourCarOrder tourCarOrderFilter,
            @Param("tourCarCategoryFilter") TourCarCategory tourCarCategoryFilter,
            @Param("tourCarDriverFilter") TourCarDriver tourCarDriverFilter,
            @Param("orderBy") String orderBy);
}
