package com.tourism.webadmin.back.dao;

import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.webadmin.back.model.TourismProject;
import org.apache.ibatis.annotations.Param;

import java.util.*;

/**
 * 旅游项目管理数据操作访问接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourismProjectMapper extends BaseDaoMapper<TourismProject> {

    /**
     * 批量插入对象列表。
     *
     * @param tourismProjectList 新增对象列表。
     */
    void insertList(List<TourismProject> tourismProjectList);

    /**
     * 获取过滤后的对象列表。
     *
     * @param tourismProjectFilter 主表过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 对象列表。
     */
    List<TourismProject> getTourismProjectList(
            @Param("tourismProjectFilter") TourismProject tourismProjectFilter, @Param("orderBy") String orderBy);

    /**
     * 根据关联主表Id，获取关联从表数据列表。
     *
     * @param travelNotesId 关联主表Id。
     * @param tourismProjectFilter 从表过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 从表数据列表。
     */
    List<TourismProject> getTourismProjectListByTravelNotesId(
            @Param("travelNotesId") String travelNotesId,
            @Param("tourismProjectFilter") TourismProject tourismProjectFilter,
            @Param("orderBy") String orderBy);

    /**
     * 根据关联主表Id列表，获取关联从表数据列表。
     *
     * @param travelNotesIds 关联主表Id列表。
     * @param tourismProjectFilter 从表过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 从表数据列表。
     */
    List<TourismProject> getTourismProjectListByTravelNotesIds(
            @Param("travelNotesIds") List<String> travelNotesIds,
            @Param("tourismProjectFilter") TourismProject tourismProjectFilter,
            @Param("orderBy") String orderBy);

    /**
     * 根据关联主表Id，获取关联从表中没有和主表建立关联关系的数据列表。
     *
     * @param travelNotesId 关联主表Id。
     * @param tourismProjectFilter 过滤对象。
     * @param orderBy 排序字符串，order by从句的参数。
     * @return 与主表没有建立关联的从表数据列表。
     */
    List<TourismProject> getNotInTourismProjectListByTravelNotesId(
            @Param("travelNotesId") String travelNotesId,
            @Param("tourismProjectFilter") TourismProject tourismProjectFilter,
            @Param("orderBy") String orderBy);
}
