package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 中国行政区划信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "中国行政区划信息Dto对象")
@Data
public class ChinaAreaDto {

    /**
     * 地区编码。
     */
    @Schema(description = "地区编码。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，地区编码不能为空！", groups = {UpdateGroup.class})
    private Long code;

    /**
     * 地区名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "地区名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，地区名称不能为空！")
    private String name;

    /**
     * 城市编码。
     */
    @Schema(description = "城市编码。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，城市编码不能为空！")
    private Long cityCode;

    /**
     * 城市名称。
     */
    @Schema(description = "城市名称。")
    private String cityName;

    /**
     * 省编码。
     */
    @Schema(description = "省编码。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，省编码不能为空！")
    private Long provinceCode;

    /**
     * 省名称。
     */
    @Schema(description = "省名称。")
    private String provinceName;

    /**
     * 国家。
     */
    @Schema(description = "国家。")
    private String country;

    /**
     * 等级 1：省,2：市,3：区县。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "等级 1：省,2：市,3：区县。可支持等于操作符的列表数据过滤。")
    private Long level;

    /**
     * name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
