package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.IsTrue;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 业务banner管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "业务banner管理Dto对象")
@Data
public class TourBusinessBannerDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * banner名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "banner名称。可支持等于操作符的列表数据过滤。")
    private String bannerName;

    /**
     * 业务名称。
     */
    @Schema(description = "业务名称。")
    private Long businessType;

    /**
     * 是否启用，0否，1是。
     */
    @Schema(description = "是否启用，0否，1是。")
    @ConstDictRef(constDictClass = IsTrue.class, message = "数据验证失败，是否启用，0否，1是为无效值！")
    private Integer enable;

    /**
     * 是否主banner，0否，1是。
     */
    @Schema(description = "是否主banner，0否，1是。")
    @ConstDictRef(constDictClass = IsTrue.class, message = "数据验证失败，是否主banner，0否，1是为无效值！")
    private Integer mainBanner;

    /**
     * 图片URL。
     */
    @Schema(description = "图片URL。")
    private String imgUrl;

    /**
     * 跳转链接。
     */
    @Schema(description = "跳转链接。")
    private String bannerUrl;

    /**
     * banner排序。
     */
    @Schema(description = "banner排序。")
    private Integer showOrder;

    /**
     * 删除标记(1: 正常 -1: 已删除)。
     */
    @Schema(description = "删除标记(1: 正常 -1: 已删除)。")
    private Integer dataState;

    /**
     * banner_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
