package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.TourCarType;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;

/**
 * 包车车型Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车车型Dto对象")
@Data
public class TourCarCategoryDto {

    /**
     * id。
     */
    @Schema(description = "id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 车型种类。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车型种类。可支持等于操作符的列表数据过滤。")
    private Long model;

    /**
     * 标题。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "标题。可支持等于操作符的列表数据过滤。")
    private String title;

    /**
     * 车名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车名。可支持等于操作符的列表数据过滤。")
    private String name;

    /**
     * 包车价格。
     */
    @Schema(description = "包车价格。")
    private BigDecimal price;

    /**
     * 单位。
     */
    @Schema(description = "单位。")
    private String unit;

    /**
     * 座位数。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "座位数。可支持等于操作符的列表数据过滤。")
    private Integer seatNumber;

    /**
     * 图片。
     */
    @Schema(description = "图片。")
    private String image;

    /**
     * 车型归属(1.平台自营;2.三方个人)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车型归属(1.平台自营;2.三方个人)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = TourCarType.class, message = "数据验证失败，车型归属(1.平台自营;2.三方个人)为无效值！")
    private Integer type;

    /**
     * 关联司机(若车型归属为2，则必填)。
     */
    @Schema(description = "关联司机(若车型归属为2，则必填)。")
    private Long relationDriverId;

    /**
     * 车型标签(以 ,隔开)。
     */
    @Schema(description = "车型标签(以 ,隔开)。")
    private String label;

    /**
     * 评分。
     */
    @Schema(description = "评分。")
    private Integer mark;

    /**
     * 组织id。
     */
    @Schema(description = "组织id。")
    private Long deptId;

    /**
     * 是否启用。
     */
    @Schema(description = "是否启用。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用为无效值！")
    private Integer enable;

    /**
     * 热度值。
     */
    @Schema(description = "热度值。")
    private Integer hotValue;

    /**
     * 所属区域。
     */
    @Schema(description = "所属区域。")
    private Long belongTab;

    /**
     * 行李数。
     */
    @Schema(description = "行李数。")
    private Integer luggageNumber;

    /**
     * price 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "price 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal priceStart;

    /**
     * price 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "price 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal priceEnd;

    /**
     * title / name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
