package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 包车区域菜单Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车区域菜单Dto对象")
@Data
public class TourCarContractMenuDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 父级id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "父级id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
//    @NotNull(message = "数据验证失败，父级id不能为空！")
    private Long parentId;

    /**
     * 菜单名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜单名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，菜单名称不能为空！")
    private String menuName;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，显示顺序不能为空！")
    private Integer showOrder;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否启用，0禁用，1启用不能为空！")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * menu_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
