package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.TourCarOrderState;

import com.tourism.webadmin.back.model.constant.TourCarType;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 包车订单Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "包车订单Dto对象")
@Data
public class TourCarOrderDto {

    /**
     * id。
     */
    @Schema(description = "id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 订单号。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "订单号。可支持等于操作符的列表数据过滤。")
    private String orderNo;

    /**
     * 包车车型id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "包车车型id。可支持等于操作符的列表数据过滤。")
    private Long categoryId;

    /**
     * 司机id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "司机id。可支持等于操作符的列表数据过滤。")
    private Long driverId;

    /**
     * 开始时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "开始时间。可支持等于操作符的列表数据过滤。")
    private String startTime;

    /**
     * 结束时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "结束时间。可支持等于操作符的列表数据过滤。")
    private String endTime;

    /**
     * 包车天数。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "包车天数。可支持等于操作符的列表数据过滤。")
    private Integer days;

    /**
     * 起点。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "起点。可支持等于操作符的列表数据过滤。")
    private Long startPlaceId;

    /**
     * 联系电话。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系电话。可支持等于操作符的列表数据过滤。")
    private String connectPhone;

    /**
     * 联系姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系姓名。可支持等于操作符的列表数据过滤。")
    private String connectName;

    /**
     * 联系人性别。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "联系人性别。可支持等于操作符的列表数据过滤。")
    private Integer connectSex;

    /**
     * 备注。
     */
    @Schema(description = "备注。")
    private String remark;

    /**
     * 订单状态(1.已下单;2.已取消;3. 已完成)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "订单状态(1.已下单;2.已取消;3. 已完成)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = TourCarOrderState.class, message = "数据验证失败，订单状态(1.已下单;2.已取消;3. 已完成)为无效值！")
    private Integer state;

    /**
     * 车型评分。
     */
    @Schema(description = "车型评分。")
    private Integer categoryMark;

    /**
     * 司机评分。
     */
    @Schema(description = "司机评分。")
    private Integer driveMark;

    /**
     * 乘车人微信。
     */
    @Schema(description = "乘车人微信。")
    private String connectWechat;

    /**
     * 司机名称。
     */
    @Schema(description = "司机名称。")
    private String driverName;

    /**
     * 司机性别。
     */
    @Schema(description = "司机性别。")
    private Integer driverSex;

    /**
     * 司机年龄。
     */
    @Schema(description = "司机年龄。")
    private Integer driverAge;

    /**
     * 司机驾龄。
     */
    @Schema(description = "司机驾龄。")
    private Integer drivingYears;

    /**
     * 司机联系方式。
     */
    @Schema(description = "司机联系方式。")
    private String driverMobile;

    /**
     * 司机照片。
     */
    @Schema(description = "司机照片。")
    private String driverImage;

    /**
     * 车型种类。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车型种类。可支持等于操作符的列表数据过滤。")
    private Long carModel;

    /**
     * 车型车名。
     */
    @Schema(description = "车型车名。")
    private String carName;

    /**
     * 车型座位数。
     */
    @Schema(description = "车型座位数。")
    private Integer carSeatNumber;

    /**
     * 车型行李数。
     */
    @Schema(description = "车型行李数。")
    private Integer carLuggageNumber;

    /**
     * 车型照片。
     */
    @Schema(description = "车型照片。")
    private String carImage;

    /**
     * 包车每天的价格。
     */
    @Schema(description = "包车每天的价格。")
    private BigDecimal dayPrice;

    /**
     * 包车每天的单位。
     */
    @Schema(description = "包车每天的单位。")
    private String dayPriceUnit;

    /**
     * 订单总价格。
     */
    @Schema(description = "订单总价格。")
    private BigDecimal totalPrice;

    /**
     * 订单总价格的单位。
     */
    @Schema(description = "订单总价格的单位。")
    private String totalPriceUnit;

    /**
     * 车型归属(1.平台自营;2.三方个人)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "车型归属(1.平台自营;2.三方个人)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = TourCarType.class, message = "数据验证失败，车型归属(1.平台自营;2.三方个人)为无效值！")
    private Integer categoryType;

    /**
     * 车主用户id。
     */
    @Schema(description = "车主用户id。")
    private Long driverSysId;

    /**
     * 车主部门id。
     */
    @Schema(description = "车主部门id。")
    private Long driverDeptId;

    /**
     * startTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "startTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String startTimeStart;

    /**
     * startTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "startTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String startTimeEnd;

    /**
     * endTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String endTimeStart;

    /**
     * endTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String endTimeEnd;

    /**
     * connect_phone / connect_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
