package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 国家地区代码表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourCountryCodeDto对象")
@Data
public class TourCountryCodeDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 英文名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "英文名称。可支持等于操作符的列表数据过滤。")
    private String englishName;

    /**
     * 国家/地区。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "国家/地区。可支持等于操作符的列表数据过滤。")
    private String country;

    /**
     * 国家代码。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "国家代码。可支持等于操作符的列表数据过滤。")
    private String countryCode;

    /**
     * 区号。
     */
    @Schema(description = "区号。")
    private String areaCode;

    /**
     * 数据状态。
     */
    @Schema(description = "数据状态。")
    private Integer dataState;

    /**
     * 拼音。
     */
    @Schema(description = "拼音。")
    private String pinYin;

    /**
     * 拼音首字母。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "拼音首字母。可支持等于操作符的列表数据过滤。")
    private String firstCode;
}
