package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.IsGoldAdvisor;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.List;

/**
 * 房屋顾问信息管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "房屋顾问信息管理Dto对象")
@Data
public class TourHouseAdvisorInfoDto {

    private String advisorIds;

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 所属开发商。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所属开发商。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，所属开发商不能为空！")
    private Long developersId;

    /**
     * 顾问类型 0.顾问 1.房东。
     */
    @Schema(description = "顾问类型 0.顾问 1.房东。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，顾问类型 0.顾问 1.房东不能为空！")
    private Integer type;

    /**
     * 姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "姓名。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，姓名不能为空！")
    private String name;

    /**
     * 行业经验年限。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "行业经验年限。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，行业经验年限不能为空！")
    private Integer experienceAge;

    /**
     * 成功交房数量。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "成功交房数量。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，成功交房数量不能为空！")
    private Integer successfulDeliveryCount;

    /**
     * 是否金牌顾问 0不是 1是。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否金牌顾问 0不是 1是。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否金牌顾问 0不是 1是不能为空！", groups = {UpdateGroup.class})
    @ConstDictRef(constDictClass = IsGoldAdvisor.class, message = "数据验证失败，是否金牌顾问 0不是 1是为无效值！")
    private Integer isGoldAdvisor;

    /**
     * 头像url。
     */
    @Schema(description = "头像url。")
    private String avatarUrl;

    /**
     * 备案编号。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "备案编号。可支持等于操作符的列表数据过滤。")
    private String recordNumber;

    /**
     * 公司名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "公司名称。可支持等于操作符的列表数据过滤。")
    private String companyName;

    /**
     * 二维码。
     */
    @Schema(description = "二维码。")
    private String code;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否启用，0禁用，1启用不能为空！")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * 手机号。
     */
    @Schema(description = "手机号。")
    private String mobile;

    /**
     * experienceAge 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "experienceAge 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer experienceAgeStart;

    /**
     * experienceAge 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "experienceAge 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer experienceAgeEnd;

    /**
     * successfulDeliveryCount 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "successfulDeliveryCount 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer successfulDeliveryCountStart;

    /**
     * successfulDeliveryCount 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "successfulDeliveryCount 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer successfulDeliveryCountEnd;

    /**
     * name / record_number / company_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
