package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;
import com.tourism.webadmin.back.model.constant.Hotspot;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 租售房区域菜单管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "租售房区域菜单管理Dto对象")
@Data
public class TourHouseAreaMenuDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 父级id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "父级id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，父级id不能为空！")
    private Long parentId;

    /**
     * 菜单等级。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜单等级。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，菜单等级不能为空！")
    private Integer levelNum;

    /**
     * 菜单名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜单名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，菜单名称不能为空！")
    private String levelName;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，显示顺序不能为空！")
    private Integer showOrder;

    /**
     * 是否启用，0禁用，1启用。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0禁用，1启用。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否启用，0禁用，1启用不能为空！")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0禁用，1启用为无效值！")
    private Integer enable;

    /**
     * 租售类型：0：出售 1：租赁。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "菜单类型：0：出售 1：租赁。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，租售类型：0：出售 1：租赁不能为空！")
    private Integer type;

    /**
     * 是否热点，0非热点，1热点。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否热点，0非热点，1热点。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Hotspot.class, message = "数据验证失败，是否热点，0非热点，1热点为无效值！")
    private Integer isHotspot;

    /**
     * 热门排序。
     */
    @Schema(description = "热门排序。")
    private Integer hotOrder;
}
