package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 房屋图集信息管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "房屋图集信息管理Dto对象")
@Data
public class TourHouseFilesDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 房屋表id。
     */
    @Schema(description = "房屋表id。", requiredMode = Schema.RequiredMode.REQUIRED)
//    @NotNull(message = "数据验证失败，房屋表id不能为空！", groups = {UpdateGroup.class})
    private Long tourHouseId;

    /**
     * 文件名称。
     */
    @Schema(description = "文件名称。")
    private String fileName;

    /**
     * 文件大小。
     */
    @Schema(description = "文件大小。")
    private String fileSize;

    /**
     * 价格。
     */
    @Schema(description = "价格。")
    private BigDecimal price;

    /**
     * 货币。
     */
    @Schema(description = "货币。")
    private String currency;

    /**
     * 面积。
     */
    @Schema(description = "面积。")
    private Integer area;

    /**
     * 文件类型 0房屋图集  1户型图集 2周边配套图集 3小区介绍图集。
     */
    @Schema(description = "文件类型 0房屋图集  1户型图集 2周边配套图集 3小区介绍图集。", requiredMode = Schema.RequiredMode.REQUIRED)
//    @NotNull(message = "数据验证失败，文件类型 0房屋图集  1户型图集 2周边配套图集 3小区介绍图集不能为空！")
    private Integer fileType;

    /**
     * 文件存储地址。
     */
    @Schema(description = "文件存储地址。")
    private String fileUrl;

    /**
     * 上传时间。
     */
    @Schema(description = "上传时间。")
    private Date uploadTime;
}
