package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.app.wechat.dto.PageInfo;
import com.tourism.webadmin.back.model.constant.RentState;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;

/**
 * 租赁房屋信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "租赁房屋信息Dto对象")
@Data
public class TourHouseRentInfoDto extends PageInfo {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 所属地区类型。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所属地区类型。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，所属地区类型不能为空！")
    private Long areaId;

    /**
     * 顾问id。
     */
    @Schema(description = "顾问id。")
    private Long advisorId;

    /**
     * 租赁方式 0整租 1合租 2个人房源 3短租。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "租赁方式 0整租 1合租 2个人房源 3短租。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，租赁方式 0整租 1合租 2个人房源 3短租不能为空！")
    private Integer rentType;

    /**
     * 厅室类型。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "厅室类型。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，厅室类型不能为空！")
    private Integer officeType;

    /**
     * 租金。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "租金。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，租金不能为空！")
    private BigDecimal rentPrice;

    /**
     * 面积。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "面积。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，面积不能为空！")
    private Integer area;

    /**
     * 房屋类型 别墅、公寓...。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "房屋类型 别墅、公寓...。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，房屋类型 别墅、公寓...不能为空！")
    private Integer houseType;

    /**
     * 租金类型(0无租金;1押一付一;2押一付二;3押一付三....)。
     */
    @Schema(description = "租金类型(0无租金;1押一付一;2押一付二;3押一付三....)。")
    private Integer rentKind;

    /**
     * 朝向类型。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "朝向类型。可支持等于操作符的列表数据过滤。")
    private Integer orientation;

    /**
     * 所在楼层。
     */
    @Schema(description = "所在楼层。")
    private Integer currentFloor;

    /**
     * 装修类型。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "装修类型。可支持等于操作符的列表数据过滤。")
    private Integer decoration;

    /**
     * 楼盘名称/所属小区。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "楼盘名称/所属小区。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，楼盘名称/所属小区不能为空！")
    private String communityName;

    /**
     * 地址。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "地址。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，地址不能为空！")
    private String address;

    /**
     * 货币单位。
     */
    @Schema(description = "货币单位。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，货币单位不能为空！")
    private String currency;

    /**
     * 所属区域。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "所属区域。可支持等于操作符的列表数据过滤。")
    private String region;

    /**
     * 首页封面图。
     */
    @Schema(description = "首页封面图。")
    private String coverUrl;

    /**
     * 房屋标题。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "房屋标题。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，房屋标题不能为空！")
    private String houseTitle;

    /**
     * 总楼层。
     */
    @Schema(description = "总楼层。")
    private Integer totalFloor;

    /**
     * 二维码。
     */
    @Schema(description = "二维码。")
    private String code;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话。")
    private String mobile;

    /**
     * 备注。
     */
    @Schema(description = "备注。")
    private String remark;

    /**
     * 排序顺序。
     */
    @Schema(description = "排序顺序。")
    private Integer showOrder;

    /**
     * 租房状态(0:未租出;1:已租出)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "租房状态(0:未租出;1:已租出)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = RentState.class, message = "数据验证失败，租房状态(0:未租出;1:已租出)为无效值！")
    private Integer rentState;

    /**
     * 楼层类型(1:低楼层;2:中楼层;3:高楼层)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "楼层类型(1:低楼层;2:中楼层;3:高楼层)。可支持等于操作符的列表数据过滤。")
    private Integer floorType;

    /**
     * rentPrice 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "rentPrice 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal rentPriceStart;

    /**
     * rentPrice 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "rentPrice 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal rentPriceEnd;

    /**
     * area 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "area 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer areaStart;

    /**
     * area 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "area 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer areaEnd;

    /**
     * community_name / address / house_title LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;

    private String areaIds;

    /**
     * 是否热推房源。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否热推房源。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = com.tourism.webadmin.back.model.constant.IsTrue.class, message = "数据验证失败，是否热推房源为无效值！")
    private Integer isHotPush;
}
