package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;

/**
 * 城市定位管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "城市定位管理Dto对象")
@Data
public class TourMapDataDto {

    /**
     * id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private String id;

    /**
     * 城市名称城市。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "城市名称城市。可支持等于操作符的列表数据过滤。")
    private String cityName;

    /**
     * 经度。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "经度。可支持等于操作符的列表数据过滤。")
    private BigDecimal lon;

    /**
     * 纬度。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "纬度。可支持等于操作符的列表数据过滤。")
    private BigDecimal lat;

    /**
     * city_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
