package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.AddGroup;
import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 消息通知表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "消息通知表Dto对象")
@Data
public class TourNoticeDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 实际业务关联对象。评论/收藏操作用户ID；游记ID等。游记ID游记ID游记ID游记ID游记ID游记ID游记ID
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "实际业务关联对象。评论/收藏操作用户ID；游记ID等。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {AddGroup.class})
    private Long relationId;

    /**
     * 消息类型：1消息回复；2收藏提醒；3游记审核。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "消息类型：1消息回复；2收藏提醒；3游记审核。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，消息类型不能为空！", groups = {AddGroup.class})
    private Integer noticeType;

    /**
     * 消息内容。
     */
    @Schema(description = "消息内容。")
    @NotBlank(message = "数据验证失败，消息内容不能为空！", groups = {AddGroup.class})
    private String noticeContent;

    /**
     * notice_type=1有值，评论ID。
     */
    @Schema(description = "notice_type=1有值，评论ID。")
    private Long commentId;

    /**
     * 通知用户。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "通知用户。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，通知用户不能为空！", groups = {AddGroup.class})
    private Long userId;

    /**
     * 通知时间。
     */
    @Schema(description = "通知时间。")
    @NotNull(message = "数据验证失败，通知时间不能为空！", groups = {AddGroup.class})
    private Date sendTime;

    /**
     * 是否已读：0未读；1已读。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否已读：0未读；1已读。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，是否已读不能为空！", groups = {AddGroup.class})
    private Integer beenRead;

    /**
     * 读取时间。
     */
    @Schema(description = "读取时间。")
    @NotNull(message = "数据验证失败，读取时间不能为空！", groups = {AddGroup.class})
    private Date readTime;
}
