package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 旅游订单游客Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourOrderPassenageDto对象")
@Data
public class TourOrderPassenageDto {

    /**
     * ID主键。
     */
    @Schema(description = "ID主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，ID主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 中文姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "中文姓名。可支持等于操作符的列表数据过滤。")
    private String chineseName;

    /**
     * 英文姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "英文姓名。可支持等于操作符的列表数据过滤。")
    private String englishName;

    /**
     * 国籍。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "国籍。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，国籍不能为空！")
    private String nationality;

    /**
     * 生日。
     */
    @Schema(description = "生日。")
    private Date birthday;

    /**
     * 手机号码。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "手机号码。可支持等于操作符的列表数据过滤。")
    private String mobile;

    /**
     * 订单号信息。
     */
    @Schema(description = "订单号信息。")
    private Long orderId;

    /**
     * chinese_name / english_name / mobile LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
