package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 开团申请列表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "开团申请列表Dto对象")
@Data
public class TourProjectGroupPurchaseApplyDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 项目id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "项目id。可支持等于操作符的列表数据过滤。")
    private Long projectId;

    /**
     * 用户Id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "用户Id。可支持等于操作符的列表数据过滤。")
    private Long tourUserId;

    /**
     * 申请时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "申请时间。可支持等于操作符的列表数据过滤。")
    private Date applyTime;

    /**
     * 是否处理(0:未处理 1:已处理)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否处理(0:未处理 1:已处理)。可支持等于操作符的列表数据过滤。")
    private Integer isHandle;

    /**
     * 备注。
     */
    @Schema(description = "备注。")
    private String remark;

    /**
     * applyTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "applyTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String applyTimeStart;

    /**
     * applyTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "applyTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String applyTimeEnd;
}
