package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;
import com.tourism.webadmin.back.model.constant.BannerType;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 拼团首页bannerDto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "拼团首页bannerDto对象")
@Data
public class TourProjectGroupPurchaseBannerDto {

    /**
     * 主键。
     */
    @Schema(description = "主键。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * banner名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "banner名称。可支持等于操作符的列表数据过滤。")
    private String bannerName;

    /**
     * 是否启用，0否，1是。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否启用，0否，1是。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，是否启用，0否，1是为无效值！")
    private Integer enable;

    /**
     * 图片URL。
     */
    @Schema(description = "图片URL。")
    private String imgUrl;

    /**
     * 旅游项目id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "旅游项目id。可支持等于操作符的列表数据过滤。")
    private Long projectId;

    /**
     * banner排序。
     */
    @Schema(description = "banner排序。")
    private Integer showOrder;

    /**
     * 类型(0.PC；1.H5)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "类型(0.PC；1.H5)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = BannerType.class, message = "数据验证失败，类型(0.PC；1.H5)为无效值！")
    private Integer type;
}
