package com.tourism.webadmin.back.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import com.tourism.common.core.validator.AddGroup;
import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.Enable;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 拼团设置主表Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "拼团设置主表Dto对象")
@Data
public class TourProjectGroupPurchaseDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 项目id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "项目id。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {AddGroup.class})
    private Long projectId;

    /**
     * 拼团结束时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "拼团结束时间。可支持等于操作符的列表数据过滤。")
    private Date endTime;

    /**
     * 出行开始时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "出行开始时间。可支持等于操作符的列表数据过滤。")
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {AddGroup.class})
    private Date travelStartTime;

    /**
     * 出行结束时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "出行结束时间。可支持等于操作符的列表数据过滤。")
    private Date travelEndTime;

    /**
     * 拼团状态(0禁用，1启用)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "拼团状态(0禁用，1启用)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = Enable.class, message = "数据验证失败，拼团状态(0禁用，1启用)为无效值！")
    private Integer state;

    /**
     * 拼团人数上限。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "拼团人数上限。可支持等于操作符的列表数据过滤。")
    private Integer maxCount;

    /**
     * 成人价格。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "成人价格。可支持等于操作符的列表数据过滤。")
    private BigDecimal adultPrice;

    /**
     * 儿童价格。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "儿童价格。可支持等于操作符的列表数据过滤。")
    private BigDecimal childrenPrice;

    /**
     * 成人原价。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "成人原价。可支持等于操作符的列表数据过滤。")
    private BigDecimal originalPrice;

    /**
     * 儿童原价。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "儿童原价。可支持等于操作符的列表数据过滤。")
    private BigDecimal originalChildPrice;

    /**
     * 显示顺序。
     */
    @Schema(description = "显示顺序。")
    private Integer showOrder;

    /**
     * 返利价格/人。
     */
    @Schema(description = "返利价格/人。")
    private BigDecimal singleRebate;

    /**
     * 关联字段(与从表)。
     */
    @Schema(description = "关联字段(与从表)。")
    private Long code;

    /**
     * 当前拼团的人数。
     */
    @Schema(description = "当前拼团的人数。")
    private Integer nowCount;

    /**
     * 拼团是否成功(0.正在拼团;1.成功)。
     */
    @Schema(description = "拼团是否成功(0.正在拼团;1.成功)。")
    private Integer success;

    /**
     * 拼团标题。
     */
    @Schema(description = "拼团标题。")
    private String title;

    /**
     * 图片url。
     */
    @Schema(description = "图片url。")
    private String image;

    /**
     * 价格单位。
     */
    @Schema(description = "价格单位。")
    private String priceUnit;

    /**
     * endTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String endTimeStart;

    /**
     * endTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "endTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String endTimeEnd;

    /**
     * travelStartTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "travelStartTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String travelStartTimeStart;

    /**
     * travelStartTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "travelStartTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String travelStartTimeEnd;

    /**
     * travelEndTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "travelEndTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String travelEndTimeStart;

    /**
     * travelEndTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "travelEndTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String travelEndTimeEnd;

    /**
     * maxCount 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "maxCount 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer maxCountStart;

    /**
     * maxCount 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "maxCount 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer maxCountEnd;

    /**
     * adultPrice 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "adultPrice 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal adultPriceStart;

    /**
     * adultPrice 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "adultPrice 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal adultPriceEnd;

    /**
     * childrenPrice 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "childrenPrice 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal childrenPriceStart;

    /**
     * childrenPrice 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "childrenPrice 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal childrenPriceEnd;

    /**
     * 当前成人价格
     */
    @Schema(description = "当前成人价格")
    private BigDecimal nowPrice;

    /**
     * 当前儿童价格
     */
    @Schema(description = "当前儿童价格")
    private BigDecimal nowChildPrice;
}
