package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.GroupPurchaseRebate;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;

/**
 * 拼团返利记录Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "拼团返利记录Dto对象")
@Data
public class TourProjectGroupPurchaseRebateDto {

    /**
     * 主键id。
     */
    @Schema(description = "主键id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 用户id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "用户id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，用户id不能为空！")
    private Long userId;

    /**
     * 拼团的id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "拼团的id。可支持等于操作符的列表数据过滤。")
    private Long groupPurchaseProgressId;

    /**
     * 订单id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "订单id。可支持等于操作符的列表数据过滤。")
    private Long orderId;

    /**
     * 返利的金额(+为增加;-为减少)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "返利的金额(+为增加;-为减少)。可支持等于操作符的列表数据过滤。")
    private BigDecimal price;

    /**
     * 返利变化的原因(0.邀请用户;1.邀请的用户取消;2.清空返利)。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "返利变化的原因(0.邀请用户;1.邀请的用户取消;2.清空返利)。可支持等于操作符的列表数据过滤。")
    @ConstDictRef(constDictClass = GroupPurchaseRebate.class, message = "数据验证失败，返利变化的原因(0.邀请用户;1.邀请的用户取消;2.清空返利)为无效值！")
    private Integer reason;

    /**
     * 返利变化的关联用户。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "返利变化的关联用户。可支持等于操作符的列表数据过滤。")
    private Long reasonRelationUserid;

    /**
     * price 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "price 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal priceStart;

    /**
     * price 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "price 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private BigDecimal priceEnd;

    /**
     * 返利金额的单位。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "返利金额的单位。")
    private BigDecimal priceUnit;
}
