package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.VisaConsoluerLevel;
import com.tourism.webadmin.back.model.constant.VisaIndexFlag;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 签证顾问信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "签证顾问信息Dto对象")
@Data
public class TourVisaCounselorDto {

    /**
     * 顾问id。
     */
    @Schema(description = "顾问id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，顾问id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 顾问头像。
     */
    @Schema(description = "顾问头像。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，顾问头像不能为空！")
    private String counselorAvatar;

    /**
     * 顾问电话。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "顾问电话。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，顾问电话不能为空！")
    private String counselorPhone;

    /**
     * 顾问微信。
     */
    @Schema(description = "顾问微信。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，顾问微信不能为空！")
    private String counselorWechat;

    /**
     * 顾问姓名。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "顾问姓名。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，顾问姓名不能为空！")
    private String counselorName;

    /**
     * 级别。
     */
    @Schema(description = "级别。")
    @ConstDictRef(constDictClass = VisaConsoluerLevel.class, message = "数据验证失败，级别为无效值！")
    private Integer level;

    /**
     * 上班时间。
     */
    @Schema(description = "上班时间。")
    private Integer start;

    /**
     * 下班时间。
     */
    @Schema(description = "下班时间。")
    private Integer end;

    /**
     * 首页显示顾问。
     */
    @Schema(description = "首页显示顾问。")
    @ConstDictRef(constDictClass = VisaIndexFlag.class, message = "数据验证失败，首页显示顾问为无效值！")
    private Integer indexFlag;

    /**
     * counselor_phone / counselor_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
