package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.InterviewFlag;
import com.tourism.webadmin.back.model.constant.Hotspot;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 签证信息管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "签证信息管理Dto对象")
@Data
public class TourVisaDto {

    /**
     * 签证id。
     */
    @Schema(description = "签证id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，签证id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 图片地址。
     */
    @Schema(description = "图片地址。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，图片地址不能为空！")
    private String img;

    /**
     * 名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，名称不能为空！")
    private String visaTitle;

    /**
     * 类型id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "类型id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，类型id不能为空！")
    private Long visaTypeId;

    /**
     * 国家编码。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "国家编码。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，国家编码不能为空！")
    private String countryCodeId;

    /**
     * 使馆名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "使馆名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，使馆名称不能为空！")
    private String embassyName;

    /**
     * 使馆地址。
     */
    @Schema(description = "使馆地址。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，使馆地址不能为空！")
    private String embassyAddress;

    /**
     * 是否面签 0 是 1否。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "是否面签 0 是 1否。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，是否面签 0 是 1否不能为空！", groups = {UpdateGroup.class})
    @ConstDictRef(constDictClass = InterviewFlag.class, message = "数据验证失败，是否面签 0 是 1否为无效值！")
    private Integer interviewFlag;

    /**
     * 有效期（日）。
     */
    @Schema(description = "有效期（日）。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，有效期（日）不能为空！", groups = {UpdateGroup.class})
    private Integer validity;

    /**
     * 停留天数。
     */
    @Schema(description = "停留天数。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，停留天数不能为空！", groups = {UpdateGroup.class})
    private Integer idleDays;

    /**
     * 预计办理天数。
     */
    @Schema(description = "预计办理天数。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，预计办理天数不能为空！", groups = {UpdateGroup.class})
    private Integer processingDays;

    /**
     * 预计价格。
     */
    @Schema(description = "预计价格。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，预计价格不能为空！", groups = {UpdateGroup.class})
    private Integer floorPrice;

    /**
     * 受理范围。
     */
    @Schema(description = "受理范围。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，受理范围不能为空！")
    private String acceptanceRange;

    /**
     * 入境次数。
     */
    @Schema(description = "入境次数。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，入境次数不能为空！", groups = {UpdateGroup.class})
    private Integer entriesNum;

    /**
     * 预定须知。
     */
    @Schema(description = "预定须知。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，预定须知不能为空！")
    private String dataText;

    /**
     * 出签地编码。
     */
    @Schema(description = "出签地编码。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，出签地编码不能为空！")
    private Long issueVisaCode;

    /**
     * 负责顾问。
     */
    @Schema(description = "负责顾问。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，负责顾问不能为空！")
    private Long counselorId;

    /**
     * 热门数据。
     */
    @Schema(description = "热门数据。")
    @ConstDictRef(constDictClass = Hotspot.class, message = "数据验证失败，热门数据为无效值！")
    private Integer hotValue;

    /**
     * visa_title / embassy_name LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
