package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 签证类型信息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "签证类型信息Dto对象")
@Data
public class TourVisaTypeDto {

    /**
     * 签证类型id。
     */
    @Schema(description = "签证类型id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，签证类型id不能为空！", groups = {UpdateGroup.class})
    private String id;

    /**
     * 签证类型名称。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "签证类型名称。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，签证类型名称不能为空！")
    private String title;

    /**
     * 签证类型介绍。
     */
    @Schema(description = "签证类型介绍。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，签证类型介绍不能为空！")
    private String recommand;

    /**
     * title LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
