package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 旅游项目日历价格Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "旅游项目日历价格Dto对象")
@Data
public class TourismDatePriceDto {

    /**
     * id。
     */
    @Schema(description = "id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 旅游项目信息。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "旅游项目信息。可支持等于操作符的列表数据过滤。")
    private Long projectId;

    /**
     * 出发日期。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "出发日期。可支持等于操作符的列表数据过滤。")
    private Date departureDate;

    /**
     * 成人价格。
     */
    @Schema(description = "成人价格。")
    private BigDecimal adultPrice;

    /**
     * 儿童价格。
     */
    @Schema(description = "儿童价格。")
    private BigDecimal childrenPrice;

    /**
     * 项目价格。
     */
    @Schema(description = "项目价格。")
    private BigDecimal price;

    /**
     * departureDate 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "departureDate 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String departureDateStart;

    /**
     * departureDate 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "departureDate 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String departureDateEnd;
}
