package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 签证基础资料管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "签证基础资料管理Dto对象")
@Data
public class VisaMetarialsListDto {

    /**
     * 基础材料id。
     */
    @Schema(description = "基础材料id。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，基础材料id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 签证商品id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "签证商品id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，签证商品id不能为空！", groups = {UpdateGroup.class})
    private Long visaId;

    /**
     * 基础材料名字。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "基础材料名字。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，基础材料名字不能为空！")
    private String title;

    /**
     * 基础材料介绍。
     */
    @Schema(description = "基础材料介绍。")
    private String content;

    /**
     * title LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
