package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.base.model.BaseModel;
import com.tourism.webadmin.back.model.constant.IsTrue;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Map;

/**
 * 业务banner管理实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_business_banner")
public class TourBusinessBanner extends BaseModel {

    /**
     * 主键。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * banner名称。
     */
    @TableField(value = "banner_name")
    private String bannerName;

    /**
     * 业务名称。
     */
    @TableField(value = "business_type")
    private Long businessType;

    /**
     * 是否启用，0否，1是。
     */
    @TableField(value = "enable")
    private Integer enable;

    /**
     * 是否主banner，0否，1是。
     */
    @TableField(value = "main_banner")
    private Integer mainBanner;

    /**
     * 图片URL。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "img_url")
    private String imgUrl;

    /**
     * 跳转链接。
     */
    @TableField(value = "banner_url")
    private String bannerUrl;

    /**
     * banner排序。
     */
    @TableField(value = "show_order")
    private Integer showOrder;

    /**
     * 删除标记(1: 正常 -1: 已删除)。
     */
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * banner_name LIKE搜索字符串。
     */
    @TableField(exist = false)
    private String searchString;

    public void setSearchString(String searchString) {
        this.searchString = MyCommonUtil.replaceSqlWildcard(searchString);
    }

    @RelationConstDict(
            masterIdField = "enable",
            constantDictClass = IsTrue.class)
    @TableField(exist = false)
    private Map<String, Object> enableDictMap;

    @RelationConstDict(
            masterIdField = "mainBanner",
            constantDictClass = IsTrue.class)
    @TableField(exist = false)
    private Map<String, Object> mainBannerDictMap;

    @RelationGlobalDict(
            masterIdField = "businessType",
            dictCode = "TourBusinessType")
    @TableField(exist = false)
    private Map<String, Object> businessTypeDictMap;
}
