package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.webadmin.upms.model.SysDept;
import com.tourism.webadmin.back.model.constant.TourCarType;
import com.tourism.webadmin.back.model.constant.Enable;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.base.model.BaseModel;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * 包车车型实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
//@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_car_category")
public class TourCarCategory {

    /**
     * id。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 车型种类(suv,mpv)。
     */
    @TableField(value = "model")
    private Long model;

    /**
     * 标题。
     */
    @TableField(value = "title")
    private String title;

    /**
     * 车名。
     */
    @TableField(value = "name")
    private String name;

    /**
     * 包车价格。
     */
    @TableField(value = "price")
    private BigDecimal price;

    /**
     * 单位。
     */
    @TableField(value = "unit")
    private String unit;

    /**
     * 座位数。
     */
    @TableField(value = "seat_number")
    private Integer seatNumber;

    /**
     * 图片。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "image")
    private String image;

    /**
     * 车型归属(1.平台自营;2.三方个人)。
     */
    @TableField(value = "type")
    private Integer type;

    /**
     * 关联司机(若车型归属为2，则必填)。
     */
    @TableField(value = "relation_driver_id")
    private Long relationDriverId;

    /**
     * 车型标签(以 ,隔开)。
     */
    @TableField(value = "label")
    private String label;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * 评分。
     */
    @TableField(value = "mark")
    private Integer mark;

    /**
     * 组织id。
     */
    @DeptFilterColumn
    @TableField(value = "dept_id")
    private Long deptId;

    /**
     * 是否启用。
     */
    @TableField(value = "enable")
    private Integer enable;

    /**
     * 热度值。
     */
    @TableField(value = "hot_value")
    private Integer hotValue;

    /**
     * 所属区域。
     */
    @TableField(value = "belong_tab")
    private Long belongTab;

    /**
     * 行李数。
     */
    @TableField(value = "luggage_number")
    private Integer luggageNumber;

    /**
     * 平均分（四舍五入）
     */
    @Schema(description = "平均分（四舍五入）")
    @TableField(exist = false)
    private Integer score;

    /**
     * price 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private BigDecimal priceStart;

    /**
     * price 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private BigDecimal priceEnd;

    /**
     * title / name LIKE搜索字符串。
     */
    @TableField(exist = false)
    private String searchString;

    public void setSearchString(String searchString) {
        this.searchString = MyCommonUtil.replaceSqlWildcard(searchString);
    }

    @RelationDict(
            masterIdField = "deptId",
            slaveModelClass = SysDept.class,
            slaveIdField = "deptId",
            slaveNameField = "deptName")
    @TableField(exist = false)
    private Map<String, Object> deptIdDictMap;

    @RelationDict(
            masterIdField = "belongTab",
            slaveModelClass = TourCarContractMenu.class,
            slaveIdField = "id",
            slaveNameField = "menuName")
    @TableField(exist = false)
    private Map<String, Object> belongTabDictMap;

    @RelationConstDict(
            masterIdField = "type",
            constantDictClass = TourCarType.class)
    @TableField(exist = false)
    private Map<String, Object> typeDictMap;

    @RelationConstDict(
            masterIdField = "enable",
            constantDictClass = Enable.class)
    @TableField(exist = false)
    private Map<String, Object> enableDictMap;

    @RelationDict(
            masterIdField = "model",
            slaveModelClass = TourCarModel.class,
            slaveIdField = "id",
            slaveNameField = "typeName")
    @TableField(exist = false)
    private Map<String, Object> modelDictMap;

    /**
     * 创建者Id。
     */
    @UserFilterColumn
    @TableField(value = "create_user_id")
    private Long createUserId;

    /**
     * 创建时间。
     */
    @TableField(value = "create_time")
    private Date createTime;

    /**
     * 更新者Id。
     */
    @TableField(value = "update_user_id")
    private Long updateUserId;

    /**
     * 更新时间。
     */
    @TableField(value = "update_time")
    private Date updateTime;
}
