package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.webadmin.back.model.constant.TourCarType;
import com.tourism.webadmin.upms.model.SysDept;
import lombok.Data;

import java.util.Date;
import java.util.Map;

/**
 * 包车司机实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
//@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_car_driver")
public class TourCarDriver {

    /**
     * id。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 司机名称。
     */
    @TableField(value = "name")
    private String name;

    /**
     * 性别。
     */
    @TableField(value = "sex")
    private Integer sex;

    /**
     * 年龄。
     */
    @TableField(value = "age")
    private Integer age;

    /**
     * 查询开始年龄
     */
    @TableField(exist = false)
    private String ageBegin;
    /**
     * 查询开始年龄
     */
    @TableField(exist = false)
    private String ageEnd;
    /**
     * 驾龄。
     */
    @TableField(value = "driving_years")
    private Integer drivingYears;

    /**
     * 司机照片。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "image")
    private String image;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * 评分。
     */
    @TableField(value = "mark")
    private Integer mark;

    /**
     * 热度值。
     */
    @TableField(value = "hot_value")
    private Integer hotValue;

    /**
     * 组织id。
     */
    @TableField(value = "dept_id")
    private Long deptId;

    /**
     * 联系方式。
     */
    @TableField(value = "mobile")
    private String mobile;

    /**
     * 司机类型(1.自营;2.第三方)。
     */
    @TableField(value = "type")
    private Integer type;

    /**
     * 所属区域。
     */
    @TableField(value = "belong_tab")
    private Long belongTab;

    /**
     *  开始日期
     */
    @TableField(exist = false)
    private String startDate;

    /**
     * 结束日期
     */
    @TableField(exist = false)
    private String endDate;

    /**
     * 平均分（四舍五入）
     */
    @TableField(exist = false)
    private Integer score;

    /**
     * name LIKE搜索字符串。
     */
    @TableField(exist = false)
    private String searchString;

    public void setSearchString(String searchString) {
        this.searchString = MyCommonUtil.replaceSqlWildcard(searchString);
    }

    @RelationOneToOne(
            masterIdField = "deptId",
            slaveModelClass = SysDept.class,
            slaveIdField = "deptId")
    @TableField(exist = false)
    private SysDept sysDept;

    @RelationConstDict(
            masterIdField = "type",
            constantDictClass = TourCarType.class)
    @TableField(exist = false)
    private Map<String, Object> typeDictMap;

    @RelationGlobalDict(
            masterIdField = "sex",
            dictCode = "UserSex")
    @TableField(exist = false)
    private Map<String, Object> sexDictMap;

    @RelationDict(
            masterIdField = "belongTab",
            slaveModelClass = TourCarContractMenu.class,
            slaveIdField = "id",
            slaveNameField = "menuName")
    @TableField(exist = false)
    private Map<String, Object> belongTabDictMap;

    /**
     * 创建者Id。
     */
    @TableField(value = "create_user_id")
    @UserFilterColumn
    private Long createUserId;

    /**
     * 创建时间。
     */
    @TableField(value = "create_time")
    private Date createTime;

    /**
     * 更新者Id。
     */
    @TableField(value = "update_user_id")
    private Long updateUserId;

    /**
     * 更新时间。
     */
    @TableField(value = "update_time")
    private Date updateTime;
}
