package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.base.model.BaseModel;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * 房屋图集信息管理实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_house_files")
public class TourHouseFiles extends BaseModel {

    /**
     * 主键。
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 房屋表id。
     */
    @TableField(value = "tour_house_id")
    private Long tourHouseId;

    /**
     * 文件名称。
     */
    @TableField(value = "file_name")
    private String fileName;

    /**
     * 文件大小。
     */
    @TableField(value = "file_size")
    private String fileSize;

    /**
     * 价格。
     */
    @TableField(value = "price")
    private BigDecimal price;

    /**
     * 货币。
     */
    @TableField(value = "currency")
    private String currency;

    /**
     * 面积。
     */
    @TableField(value = "area")
    private Integer area;

    /**
     * 文件类型 0房屋图集  1户型图集 2周边配套图集 3小区介绍图集。
     */
    @TableField(value = "file_type")
    private Integer fileType;

    /**
     * 文件存储地址。
     */
    @UploadFlagColumn(storeType = UploadStoreTypeEnum.HUAWEI_OBS_SYSTEM)
    @TableField(value = "file_url")
    private String fileUrl;

    /**
     * 上传时间。
     */
    @TableField(value = "upload_time")
    private Date uploadTime;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    @RelationGlobalDict(
            masterIdField = "fileType",
            dictCode = "HouseFileType")
    @TableField(exist = false)
    private Map<String, Object> fileTypeDictMap;
}
