package com.tourism.webadmin.back.model;

import com.baomidou.mybatisplus.annotation.*;
import com.tourism.common.core.annotation.*;
import com.tourism.common.core.base.model.BaseModel;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.List;

/**
 * 拼团设置从表实体对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "tour_project_group_purchase_detail")
public class TourProjectGroupPurchaseDetail extends BaseModel {

    /**
     * 主键id。
     */
    @TableId(value = "id")
    private Long id;

    /**
     * 关联 拼团设置的code。
     */
    @TableField(value = "group_purchase_code")
    private Long groupPurchaseCode;

    /**
     * 逻辑删除标记字段(1: 正常 -1: 已删除)。
     */
    @TableLogic
    @TableField(value = "data_state")
    private Integer dataState;

    /**
     * 成人价格。
     */
    @TableField(value = "adult_price")
    private BigDecimal adultPrice;

    /**
     * 儿童价格。
     */
    @TableField(value = "children_price")
    private BigDecimal childrenPrice;

    /**
     * 阶梯的人数(下限)。
     */
    @TableField(value = "min_count")
    private Integer minCount;

    /**
     * 阶梯的人数(上限)。
     */
    @TableField(value = "max_count")
    private Integer maxCount;

    /**
     * adultPrice 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private BigDecimal adultPriceStart;

    /**
     * adultPrice 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private BigDecimal adultPriceEnd;

    /**
     * childrenPrice 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private BigDecimal childrenPriceStart;

    /**
     * childrenPrice 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private BigDecimal childrenPriceEnd;

    /**
     * minCount 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private Integer minCountStart;

    /**
     * minCount 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private Integer minCountEnd;

    /**
     * maxCount 范围过滤起始值(>=)。
     */
    @TableField(exist = false)
    private Integer maxCountStart;

    /**
     * maxCount 范围过滤结束值(<=)。
     */
    @TableField(exist = false)
    private Integer maxCountEnd;

    @TableField(exist = false)
    private String priceUnit;
}
