package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 审核状态常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class AuditState {

    /**
     * 待审核。
     */
    public static final int UNAUDITSTATE = 1;
    /**
     * 审核未通过。
     */
    public static final int FAILAUDIT = 2;
    /**
     * 审核通过。
     */
    public static final int SUCCEEDAUDIT = 3;
    /**
     * 草稿。
     */
    public static final int DRAFT = 0;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(4);
    static {
        DICT_MAP.put(UNAUDITSTATE, "待审核");
        DICT_MAP.put(FAILAUDIT, "审核未通过");
        DICT_MAP.put(SUCCEEDAUDIT, "审核通过");
        DICT_MAP.put(DRAFT, "草稿");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private AuditState() {
    }
}
