package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 是否启用常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class BannerType {

    /**
     * PC。
     */
    public static final int BANNER_PC = 0;
    /**
     * H5。
     */
    public static final int BANNER_H5 = 1;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(2);
    static {
        DICT_MAP.put(BANNER_PC, "PC");
        DICT_MAP.put(BANNER_H5, "H5");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private BannerType() {
    }
}
