package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 租房状态常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class RentState {

    /**
     * 未租出。
     */
    public static final int ON_LEASE = 0;
    /**
     * 已租出。
     */
    public static final int RENTED_OUT = 1;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(2);
    static {
        DICT_MAP.put(ON_LEASE, "未租出");
        DICT_MAP.put(RENTED_OUT, "已租出");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private RentState() {
    }
}
