package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 包车订单状态常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class TourCarOrderState {

    /**
     * 已下单。
     */
    public static final int PLACED_ORDER = 1;
    /**
     * 已取消。
     */
    public static final int CANCELLED = 2;
    /**
     * 已完成。
     */
    public static final int COMPLETED = 3;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(3);
    static {
        DICT_MAP.put(PLACED_ORDER, "已下单");
        DICT_MAP.put(CANCELLED, "已取消");
        DICT_MAP.put(COMPLETED, "已完成");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private TourCarOrderState() {
    }
}
