package com.tourism.webadmin.app.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 消息类型常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class TourNoticeType {

    /**
     * 消息回复。
     */
    public static final int NOTICE_REPLY = 1;
    /**
     * 收藏提醒。
     */
    public static final int COLLECTION_REMINDER = 2;
    /**
     * 游记审核。
     */
    public static final int TRAVEL_REVIEW = 3;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(3);
    static {
        DICT_MAP.put(NOTICE_REPLY, "消息回复");
        DICT_MAP.put(COLLECTION_REMINDER, "收藏提醒");
        DICT_MAP.put(TRAVEL_REVIEW, "游记审核");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private TourNoticeType() {
    }
}
