package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 签证订单状态常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class VisaOrderState {

    /**
     * 材料初审。
     */
    public static final int VISAORDERSTATE_CREATE = 0;
    /**
     * 递交使馆。
     */
    public static final int VISAORDERSTATE_UP = 1;
    /**
     * 成功下签。
     */
    public static final int VISAORDERSTATE_FINSH = 2;
    /**
     * 订单已取消。
     */
    public static final int VISAORDERSTATE_CANCEL = 3;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(4);
    static {
        DICT_MAP.put(VISAORDERSTATE_CREATE, "材料初审");
        DICT_MAP.put(VISAORDERSTATE_UP, "递交使馆");
        DICT_MAP.put(VISAORDERSTATE_FINSH, "成功下签");
        DICT_MAP.put(VISAORDERSTATE_CANCEL, "订单已取消");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private VisaOrderState() {
    }
}
