package com.tourism.webadmin.back.model.constant;

import java.util.HashMap;
import java.util.Map;

/**
 * 签证支付状态常量字典对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public final class VisaPayState {

    /**
     * 未支付。
     */
    public static final int VISAPAY_NO = 0;
    /**
     * 已支付。
     */
    public static final int VISAPAY_YES = 1;
    /**
     * 已退款。
     */
    public static final int VISAPAY_BACK = 2;

    private static final Map<Object, String> DICT_MAP = new HashMap<>(3);
    static {
        DICT_MAP.put(VISAPAY_NO, "未支付");
        DICT_MAP.put(VISAPAY_YES, "已支付");
        DICT_MAP.put(VISAPAY_BACK, "已退款");
    }

    /**
     * 判断参数是否为当前常量字典的合法值。
     *
     * @param value 待验证的参数值。
     * @return 合法返回true，否则false。
     */
    public static boolean isValid(Integer value) {
        return value != null && DICT_MAP.containsKey(value);
    }

    /**
     * 私有构造函数，明确标识该常量类的作用。
     */
    private VisaPayState() {
    }
}
