package com.tourism.webadmin.back.service;

import com.tourism.common.core.base.service.IBaseService;
import com.tourism.webadmin.back.model.TourCarCategory;
import com.tourism.webadmin.back.model.TourCarDriver;
import com.tourism.webadmin.back.model.TourCarOrder;

import java.util.List;

/**
 * 包车订单数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourCarOrderService extends IBaseService<TourCarOrder, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourCarOrder 新增对象。
     * @return 返回新增对象。
     */
    TourCarOrder saveNew(TourCarOrder tourCarOrder);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourCarOrderList 新增对象列表。
     */
    void saveNewBatch(List<TourCarOrder> tourCarOrderList);

    /**
     * 更新数据对象。
     *
     * @param tourCarOrder         更新的对象。
     * @param originalTourCarOrder 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourCarOrder tourCarOrder, TourCarOrder originalTourCarOrder);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourCarOrderListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourCarOrder> getTourCarOrderList(TourCarOrder filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourCarOrderList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param tourCarCategoryFilter 一对一从表过滤对象。
     * @param tourCarDriverFilter 一对一从表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourCarOrder> getTourCarOrderListWithRelation(TourCarOrder filter, TourCarCategory tourCarCategoryFilter, TourCarDriver tourCarDriverFilter, String orderBy);

    /**
     * 判断司机在某时间段内有没有被预定
     * @param tourCarOrder
     * @return
     */
    boolean verifyDriverOrder(TourCarOrder tourCarOrder);
}
