package com.tourism.webadmin.back.service;

import com.tourism.common.core.base.service.IBaseService;
import com.tourism.webadmin.back.model.TourCountryCode;

import java.util.*;

/**
 * 国家地区代码表数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourCountryCodeService extends IBaseService<TourCountryCode, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourCountryCode 新增对象。
     * @return 返回新增对象。
     */
    TourCountryCode saveNew(TourCountryCode tourCountryCode);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourCountryCodeList 新增对象列表。
     */
    void saveNewBatch(List<TourCountryCode> tourCountryCodeList);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。通常适用于更大的插入数据量，如批量导入。
     *
     * @param tourCountryCodeList 新增对象列表。
     * @param batchSize  每批插入的数量。如果该值小于等于0，则使用缺省值10000。
     */
    void saveNewBatch(List<TourCountryCode> tourCountryCodeList, int batchSize);

    /**
     * 更新数据对象。
     *
     * @param tourCountryCode         更新的对象。
     * @param originalTourCountryCode 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourCountryCode tourCountryCode, TourCountryCode originalTourCountryCode);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourCountryCodeListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourCountryCode> getTourCountryCodeList(TourCountryCode filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourCountryCodeList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourCountryCode> getTourCountryCodeListWithRelation(TourCountryCode filter, String orderBy);

    public void updateBatch(List<TourCountryCode> tourCountryCodeList);

    List<Map> selectByIds(Set<Long> collect);
}
