package com.tourism.webadmin.back.service;

import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.IBaseService;

import java.util.*;

/**
 * 房屋顾问信息管理数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourHouseAdvisorInfoService extends IBaseService<TourHouseAdvisorInfo, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourHouseAdvisorInfo 新增对象。
     * @return 返回新增对象。
     */
    TourHouseAdvisorInfo saveNew(TourHouseAdvisorInfo tourHouseAdvisorInfo);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourHouseAdvisorInfoList 新增对象列表。
     */
    void saveNewBatch(List<TourHouseAdvisorInfo> tourHouseAdvisorInfoList);

    /**
     * 更新数据对象。
     *
     * @param tourHouseAdvisorInfo         更新的对象。
     * @param originalTourHouseAdvisorInfo 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourHouseAdvisorInfo tourHouseAdvisorInfo, TourHouseAdvisorInfo originalTourHouseAdvisorInfo);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourHouseAdvisorInfoListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoList(TourHouseAdvisorInfo filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourHouseAdvisorInfoList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoListWithRelation(TourHouseAdvisorInfo filter, String orderBy);

    /**
     * 在多对多关系中，当前Service的数据表为从表，返回不与指定主表主键Id存在对多对关系的列表。
     *
     * @param houseId 主表主键Id。
     * @param filter 从表的过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseAdvisorInfo> getNotInTourHouseAdvisorInfoListByHouseId(Long houseId, TourHouseAdvisorInfo filter, String orderBy);

    /**
     * 在多对多关系中，当前Service的数据表为从表，返回与指定主表主键Id存在对多对关系的列表。
     *
     * @param houseId 主表主键Id。
     * @param filter 从表的过滤对象。
     * @param tourHouseAdvisorRelationFilter 多对多关联表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoListByHouseId(
            Long houseId, TourHouseAdvisorInfo filter, TourHouseAdvisorRelation tourHouseAdvisorRelationFilter, String orderBy);

    /**
     * 在多对多关系中，当前Service的数据表为从表，返回与指定主表主键Id列表存在对多对关系的列表。
     *
     * @param tourHouseSaleInfoList 主表对象集合。
     * @param filter 从表的过滤对象。
     * @param orderBy 排序参数。
     * @return key是主表的主键Id，value是多对多从表的对象列表。
     */
    Map<Long, List<TourHouseAdvisorInfo>> getTourHouseAdvisorInfoListMapByTourHouseSaleInfoList(
            List<TourHouseSaleInfo> tourHouseSaleInfoList, TourHouseAdvisorInfo filter, String orderBy);

    List<TourHouseAdvisorInfo> getAdvisors(Long id);
}
