package com.tourism.webadmin.back.service;

import com.alibaba.fastjson.JSONObject;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.service.IBaseService;

import java.util.*;

/**
 * 租赁房屋信息数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourHouseRentInfoService extends IBaseService<TourHouseRentInfo, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourHouseRentInfo 新增对象。
     * @return 返回新增对象。
     */
    TourHouseRentInfo saveNew(TourHouseRentInfo tourHouseRentInfo);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourHouseRentInfoList 新增对象列表。
     */
    void saveNewBatch(List<TourHouseRentInfo> tourHouseRentInfoList);

    /**
     * 保存新增主表对象及关联对象。
     *
     * @param tourHouseRentInfo 新增主表对象。
     * @param relationData 全部关联从表数据。
     * @return 返回新增主表对象。
     */
    TourHouseRentInfo saveNewWithRelation(TourHouseRentInfo tourHouseRentInfo, JSONObject relationData);

    /**
     * 更新数据对象。
     *
     * @param tourHouseRentInfo         更新的对象。
     * @param originalTourHouseRentInfo 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourHouseRentInfo tourHouseRentInfo, TourHouseRentInfo originalTourHouseRentInfo);

    /**
     * 更新主表对象及关联对象。
     *
     * @param tourHouseRentInfo 主表对象新数据。
     * @param originalTourHouseRentInfo 主表对象源数据。
     * @param relationData 全部关联从表数据。
     * @return 修改成功返回true，否则false。
     */
    boolean updateWithRelation(TourHouseRentInfo tourHouseRentInfo, TourHouseRentInfo originalTourHouseRentInfo, JSONObject relationData);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourHouseRentInfoListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseRentInfo> getTourHouseRentInfoList(TourHouseRentInfo filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourHouseRentInfoList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param tourHouseEquipmentFilter 一对一从表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourHouseRentInfo> getTourHouseRentInfoListWithRelation(TourHouseRentInfo filter, TourHouseEquipment tourHouseEquipmentFilter, String orderBy);

    List<TourHouseRentInfo> getTourHouseRentInfoListWithArea(TourHouseRentInfo filter, String orderBy);
}
