package com.tourism.webadmin.back.service;

import com.tourism.common.core.base.service.IBaseService;
import com.tourism.webadmin.app.website.dto.TourGroupUserToWebDto;
import com.tourism.webadmin.back.dto.TourOrderDto;
import com.tourism.webadmin.back.model.TourOrder;
import com.tourism.webadmin.back.model.TourUser;

import java.util.List;

/**
 * 旅游订单数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourOrderService extends IBaseService<TourOrder, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourOrder 新增对象。
     * @return 返回新增对象。
     */
    TourOrder saveNew(TourOrder tourOrder);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourOrderList 新增对象列表。
     */
    void saveNewBatch(List<TourOrder> tourOrderList);

    /**
     * 更新数据对象。
     *
     * @param tourOrder         更新的对象。
     * @param originalTourOrder 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourOrder tourOrder, TourOrder originalTourOrder);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourOrderListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourOrder> getTourOrderList(TourOrder filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourOrderList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourOrder> getTourOrderListWithRelation(TourOrder filter, String orderBy);

    /**
     * 根据项目id查询该项目参与拼团的用户
     * @param projectId
     * @return
     */
    List<TourUser> getTourUserListByProjectId(TourGroupUserToWebDto dto);

    /**
     * 更新参与拼团的其他订单价格
     * @param tourOrder
     */
    void updateHistoryOrderPrice(TourOrder tourOrder);
}
