package com.tourism.webadmin.back.service;

import com.tourism.common.core.base.service.IBaseService;
import com.tourism.common.core.object.CallResult;
import com.tourism.webadmin.back.model.TourProjectGroupPurchase;
import com.tourism.webadmin.back.model.TourProjectGroupPurchaseDetail;

import java.util.List;

/**
 * 拼团设置从表数据操作服务接口。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
public interface TourProjectGroupPurchaseDetailService extends IBaseService<TourProjectGroupPurchaseDetail, Long> {

    /**
     * 保存新增对象。
     *
     * @param tourProjectGroupPurchaseDetail 新增对象。
     * @return 返回新增对象。
     */
    TourProjectGroupPurchaseDetail saveNew(TourProjectGroupPurchaseDetail tourProjectGroupPurchaseDetail);

    /**
     * 利用数据库的insertList语法，批量插入对象列表。
     *
     * @param tourProjectGroupPurchaseDetailList 新增对象列表。
     */
    void saveNewBatch(List<TourProjectGroupPurchaseDetail> tourProjectGroupPurchaseDetailList);

    /**
     * 更新数据对象。
     *
     * @param tourProjectGroupPurchaseDetail         更新的对象。
     * @param originalTourProjectGroupPurchaseDetail 原有数据对象。
     * @return 成功返回true，否则false。
     */
    boolean update(TourProjectGroupPurchaseDetail tourProjectGroupPurchaseDetail, TourProjectGroupPurchaseDetail originalTourProjectGroupPurchaseDetail);

    /**
     * 删除指定数据。
     *
     * @param id 主键Id。
     * @return 成功返回true，否则false。
     */
    boolean remove(Long id);

    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getTourProjectGroupPurchaseDetailListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourProjectGroupPurchaseDetail> getTourProjectGroupPurchaseDetailList(TourProjectGroupPurchaseDetail filter, String orderBy);

    /**
     * 获取主表的查询结果，以及主表关联的字典数据和一对一从表数据，以及一对一从表的字典数据。
     * 该查询会涉及到一对一从表的关联过滤，或一对多从表的嵌套关联过滤，因此性能不如单表过滤。
     * 如果仅仅需要获取主表数据，请移步(getTourProjectGroupPurchaseDetailList)，以便获取更好的查询性能。
     *
     * @param filter 主表过滤对象。
     * @param tourProjectGroupPurchaseFilter 一对多从表过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    List<TourProjectGroupPurchaseDetail> getTourProjectGroupPurchaseDetailListWithRelation(TourProjectGroupPurchaseDetail filter, TourProjectGroupPurchase tourProjectGroupPurchaseFilter, String orderBy);

    /**
     * 判断从表数据是否存在，如果存在就不能删除主对象，否则可以删除主对象。
     * 适用于主表对从表不是强制级联删除的场景。
     *
     * @param tourProjectGroupPurchaseDetail 主表对象。
     * @return 没有关联数据返回true，否则false，同时返回具体的提示信息。
     */
    CallResult verifyRelatedDataBeforeDelete(TourProjectGroupPurchaseDetail tourProjectGroupPurchaseDetail);

    /**
     * 查询团购当前阶梯价格
     * @param tourProjectGroupPurchase
     * @return
     */
    TourProjectGroupPurchaseDetail getTourProjectGroupPurchaseDetail(TourProjectGroupPurchase tourProjectGroupPurchase);

    /**
     * 查询当前阶段价格和下一阶段价格
     * @param tourProjectGroupPurchase
     * @return
     */
    TourProjectGroupPurchase getNowPriceAndNextStagePrice(TourProjectGroupPurchase tourProjectGroupPurchase);
}
