package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.webadmin.upms.service.SysDeptService;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 包车车型数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourCarCategoryService")
public class TourCarCategoryServiceImpl extends BaseService<TourCarCategory, Long> implements TourCarCategoryService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourCarCategoryMapper tourCarCategoryMapper;
    @Autowired
    private TourCarModelService tourCarModelService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private TourCarContractMenuService tourCarContractMenuService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCarCategory> mapper() {
        return tourCarCategoryMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCarCategory saveNew(TourCarCategory tourCarCategory) {
        tourCarCategoryMapper.insert(this.buildDefaultValue(tourCarCategory));
        return tourCarCategory;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarCategory> tourCarCategoryList) {
        if (CollUtil.isNotEmpty(tourCarCategoryList)) {
            tourCarCategoryList.forEach(this::buildDefaultValue);
            tourCarCategoryMapper.insertList(tourCarCategoryList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCarCategory tourCarCategory, TourCarCategory originalTourCarCategory) {
        MyModelUtil.fillCommonsForUpdate(tourCarCategory, originalTourCarCategory);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCarCategory> uw = this.createUpdateQueryForNullValue(tourCarCategory, tourCarCategory.getId());
        return tourCarCategoryMapper.update(tourCarCategory, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCarCategoryMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCarCategory> getTourCarCategoryList(TourCarCategory filter, String orderBy) {
        return tourCarCategoryMapper.getTourCarCategoryList(filter, orderBy);
    }

    @Override
    public List<TourCarCategory> getTourCarCategoryListWithRelation(TourCarCategory filter, TourCarDriver tourCarDriverFilter, String orderBy) {
        List<TourCarCategory> resultList =
                tourCarCategoryMapper.getTourCarCategoryListEx(filter, tourCarDriverFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourCarCategory tourCarCategory, TourCarCategory originalTourCarCategory) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarCategory, originalTourCarCategory, TourCarCategory::getModel)
                && !tourCarModelService.existId(tourCarCategory.getModel())) {
            return CallResult.error(String.format(errorMessageFormat, "车型种类(suv,mpv)"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarCategory, originalTourCarCategory, TourCarCategory::getDeptId)
                && !sysDeptService.existId(tourCarCategory.getDeptId())) {
            return CallResult.error(String.format(errorMessageFormat, "组织id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarCategory, originalTourCarCategory, TourCarCategory::getBelongTab)
                && !tourCarContractMenuService.existId(tourCarCategory.getBelongTab())) {
            return CallResult.error(String.format(errorMessageFormat, "所属区域"));
        }
        return CallResult.ok();
    }

    private TourCarCategory buildDefaultValue(TourCarCategory tourCarCategory) {
        if (tourCarCategory.getId() == null) {
            tourCarCategory.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourCarCategory);
        tourCarCategory.setDataState(GlobalDeletedFlag.NORMAL);
        return tourCarCategory;
    }
}
