package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.vo.TourCarContractMenuVo;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 包车区域菜单数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourCarContractMenuService")
public class TourCarContractMenuServiceImpl extends BaseService<TourCarContractMenu, Long> implements TourCarContractMenuService {

    @Autowired
    private TourCarContractMenuMapper tourCarContractMenuMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCarContractMenu> mapper() {
        return tourCarContractMenuMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCarContractMenu saveNew(TourCarContractMenu tourCarContractMenu) {
        tourCarContractMenuMapper.insert(this.buildDefaultValue(tourCarContractMenu));
        return tourCarContractMenu;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarContractMenu> tourCarContractMenuList) {
        if (CollUtil.isNotEmpty(tourCarContractMenuList)) {
            tourCarContractMenuList.forEach(this::buildDefaultValue);
            tourCarContractMenuMapper.insertList(tourCarContractMenuList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCarContractMenu tourCarContractMenu, TourCarContractMenu originalTourCarContractMenu) {
        MyModelUtil.fillCommonsForUpdate(tourCarContractMenu, originalTourCarContractMenu);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCarContractMenu> uw = this.createUpdateQueryForNullValue(tourCarContractMenu, tourCarContractMenu.getId());
        return tourCarContractMenuMapper.update(tourCarContractMenu, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCarContractMenuMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCarContractMenu> getTourCarContractMenuList(TourCarContractMenu filter, String orderBy) {
        return tourCarContractMenuMapper.getTourCarContractMenuList(filter, orderBy);
    }

    @Override
    public List<TourCarContractMenu> getTourCarContractMenuListWithRelation(TourCarContractMenu filter, String orderBy) {
        List<TourCarContractMenu> resultList = tourCarContractMenuMapper.getTourCarContractMenuList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public boolean hasChildren(Long id) {
        TourCarContractMenu filter = new TourCarContractMenu();
        filter.setParentId(id);
        return getCountByFilter(filter) > 0;
    }

    private TourCarContractMenu buildDefaultValue(TourCarContractMenu tourCarContractMenu) {
        MyModelUtil.fillCommonsForInsert(tourCarContractMenu);
        tourCarContractMenu.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourCarContractMenu, "menuName", "");
        return tourCarContractMenu;
    }



    //区域树
    @Override
    public List<TourCarContractMenuVo> getCarContractMenueTree() throws Exception {
        TourCarContractMenu filter = new TourCarContractMenu();
        filter.setEnable(1);
        filter.setDataState(1);
        List<TourCarContractMenu> allMenu = this.getTourCarContractMenuList(filter, null);
        // 所有的确按照父级分组
        Map<Long, List<TourCarContractMenu>> collect = allMenu.stream().filter(e ->e.getParentId() != null).collect(Collectors.groupingBy(TourCarContractMenu::getParentId));
        // 获取所有一级地区
        List<TourCarContractMenu> list = allMenu.stream().filter(e -> e.getParentId() != null && e.getParentId() == 0l).toList();
        List<TourCarContractMenuVo> resultList = MyModelUtil.copyCollectionTo(list, TourCarContractMenuVo.class);
        this.findChildren(resultList, collect);
        return resultList;
    }

    /**
     * 递归填充子区域
     * @param resultList
     * @param collect
     */
    private void findChildren(List<TourCarContractMenuVo> resultList, Map<Long, List<TourCarContractMenu>> collect){
        for (TourCarContractMenuVo menuVo : resultList) {
            List<TourCarContractMenu> childrens = collect.get(Long.parseLong(menuVo.getId()));
            if(childrens != null && childrens.size() > 0){
                List<TourCarContractMenuVo> childrenVos = MyModelUtil.copyCollectionTo(childrens, TourCarContractMenuVo.class);
                menuVo.setChildren(childrenVos);
                this.findChildren(childrenVos, collect);
            }
        }
    }
}
