package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 包车司机数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourCarDriverService")
public class TourCarDriverServiceImpl extends BaseService<TourCarDriver, Long> implements TourCarDriverService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourCarDriverMapper tourCarDriverMapper;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCarDriver> mapper() {
        return tourCarDriverMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCarDriver saveNew(TourCarDriver tourCarDriver) {
        tourCarDriverMapper.insert(this.buildDefaultValue(tourCarDriver));
        return tourCarDriver;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarDriver> tourCarDriverList) {
        if (CollUtil.isNotEmpty(tourCarDriverList)) {
            tourCarDriverList.forEach(this::buildDefaultValue);
            tourCarDriverMapper.insertList(tourCarDriverList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCarDriver tourCarDriver, TourCarDriver originalTourCarDriver) {
        MyModelUtil.fillCommonsForUpdate(tourCarDriver, originalTourCarDriver);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCarDriver> uw = this.createUpdateQueryForNullValue(tourCarDriver, tourCarDriver.getId());
        return tourCarDriverMapper.update(tourCarDriver, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCarDriverMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCarDriver> getTourCarDriverList(TourCarDriver filter, String orderBy) {
        return tourCarDriverMapper.getTourCarDriverList(filter, orderBy);
    }

    @Override
    public List<TourCarDriver> getTourCarDriverListWithRelation(TourCarDriver filter, String orderBy) {
        List<TourCarDriver> resultList = tourCarDriverMapper.getTourCarDriverList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourCarDriver tourCarDriver, TourCarDriver originalTourCarDriver) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarDriver, originalTourCarDriver, TourCarDriver::getSex)
                && !globalDictService.existDictItemFromCache("UserSex", tourCarDriver.getSex())) {
            return CallResult.error(String.format(errorMessageFormat, "性别"));
        }
        return CallResult.ok();
    }

    private TourCarDriver buildDefaultValue(TourCarDriver tourCarDriver) {
        if (tourCarDriver.getId() == null) {
            tourCarDriver.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourCarDriver);
        tourCarDriver.setDataState(GlobalDeletedFlag.NORMAL);
        return tourCarDriver;
    }
}
