package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 包车车型数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourCarModelService")
public class TourCarModelServiceImpl extends BaseService<TourCarModel, Long> implements TourCarModelService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourCarModelMapper tourCarModelMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCarModel> mapper() {
        return tourCarModelMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCarModel saveNew(TourCarModel tourCarModel) {
        tourCarModelMapper.insert(this.buildDefaultValue(tourCarModel));
        return tourCarModel;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarModel> tourCarModelList) {
        if (CollUtil.isNotEmpty(tourCarModelList)) {
            tourCarModelList.forEach(this::buildDefaultValue);
            tourCarModelMapper.insertList(tourCarModelList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarModel> tourCarModelList, int batchSize) {
        if (CollUtil.isEmpty(tourCarModelList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourCarModelList.size(), start + batchSize);
            List<TourCarModel> subList = tourCarModelList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourCarModelMapper.insertList(subList);
            if (end == tourCarModelList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCarModel tourCarModel, TourCarModel originalTourCarModel) {
        MyModelUtil.fillCommonsForUpdate(tourCarModel, originalTourCarModel);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCarModel> uw = this.createUpdateQueryForNullValue(tourCarModel, tourCarModel.getId());
        return tourCarModelMapper.update(tourCarModel, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCarModelMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCarModel> getTourCarModelList(TourCarModel filter, String orderBy) {
        return tourCarModelMapper.getTourCarModelList(filter, orderBy);
    }

    @Override
    public List<TourCarModel> getTourCarModelListWithRelation(TourCarModel filter, String orderBy) {
        List<TourCarModel> resultList = tourCarModelMapper.getTourCarModelList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourCarModel> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", TourCarModel::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    private TourCarModel buildDefaultValue(TourCarModel tourCarModel) {
        if (tourCarModel.getId() == null) {
            tourCarModel.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourCarModel);
        return tourCarModel;
    }
}
