package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.webadmin.back.dao.TourCountryCodeMapper;
import com.tourism.webadmin.back.model.TourCountryCode;
import com.tourism.webadmin.back.service.TourCountryCodeService;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 国家地区代码表数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourCountryCodeService")
public class TourCountryCodeServiceImpl extends BaseService<TourCountryCode, Long> implements TourCountryCodeService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourCountryCodeMapper tourCountryCodeMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCountryCode> mapper() {
        return tourCountryCodeMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCountryCode saveNew(TourCountryCode tourCountryCode) {
        tourCountryCodeMapper.insert(this.buildDefaultValue(tourCountryCode));
        return tourCountryCode;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCountryCode> tourCountryCodeList) {
        if (CollUtil.isNotEmpty(tourCountryCodeList)) {
            tourCountryCodeList.forEach(this::buildDefaultValue);
            tourCountryCodeMapper.insertList(tourCountryCodeList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCountryCode> tourCountryCodeList, int batchSize) {
        if (CollUtil.isEmpty(tourCountryCodeList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourCountryCodeList.size(), start + batchSize);
            List<TourCountryCode> subList = tourCountryCodeList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourCountryCodeMapper.insertList(subList);
            if (end == tourCountryCodeList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCountryCode tourCountryCode, TourCountryCode originalTourCountryCode) {
        MyModelUtil.fillCommonsForUpdate(tourCountryCode, originalTourCountryCode);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCountryCode> uw = this.createUpdateQueryForNullValue(tourCountryCode, tourCountryCode.getId());
        return tourCountryCodeMapper.update(tourCountryCode, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCountryCodeMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCountryCode> getTourCountryCodeList(TourCountryCode filter, String orderBy) {
        return tourCountryCodeMapper.getTourCountryCodeList(filter, orderBy);
    }

    @Override
    public List<TourCountryCode> getTourCountryCodeListWithRelation(TourCountryCode filter, String orderBy) {
        List<TourCountryCode> resultList = tourCountryCodeMapper.getTourCountryCodeList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
//        tourCountryCodeMapper.up
        return resultList;
    }

    private TourCountryCode buildDefaultValue(TourCountryCode tourCountryCode) {
        if (tourCountryCode.getId() == null) {
            tourCountryCode.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourCountryCode);
        MyModelUtil.setDefaultValue(tourCountryCode, "pinYin", "");
        MyModelUtil.setDefaultValue(tourCountryCode, "firstCode", "");
        return tourCountryCode;
    }

    @Override
    public void updateBatch(List<TourCountryCode> tourCountryCodeList) {
        //更新表数据
        tourCountryCodeList.forEach(e->{
            tourCountryCodeMapper.updateById(e);
        });
    }

    @Override
    public List<Map> selectByIds(Set<Long> collect) {
        return tourCountryCodeMapper.selectUrlById(collect);
    }
}
