package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.model.constant.IsGoldAdvisor;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 房屋顾问信息管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
//@MyDataSource(DataSourceType.REAL_TEST)
@Service("tourHouseAdvisorInfoService")
public class TourHouseAdvisorInfoServiceImpl extends BaseService<TourHouseAdvisorInfo, Long> implements TourHouseAdvisorInfoService {

    @Autowired
    private TourHouseAdvisorInfoMapper tourHouseAdvisorInfoMapper;
    @Autowired
    private TourHouseAdvisorRelationMapper tourHouseAdvisorRelationMapper;
    @Autowired
    private TourHouseDevelopersTypeService tourHouseDevelopersTypeService;
    @Autowired
    private TourHouseSaleInfoService tourHouseSaleInfoService;
//    @Autowired
//    private TourHouseAdvisorRelationService tourHouseAdvisorRelationService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseAdvisorInfo> mapper() {
        return tourHouseAdvisorInfoMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseAdvisorInfo saveNew(TourHouseAdvisorInfo tourHouseAdvisorInfo) {
        tourHouseAdvisorInfoMapper.insert(this.buildDefaultValue(tourHouseAdvisorInfo));
        return tourHouseAdvisorInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseAdvisorInfo> tourHouseAdvisorInfoList) {
        if (CollUtil.isNotEmpty(tourHouseAdvisorInfoList)) {
            tourHouseAdvisorInfoList.forEach(this::buildDefaultValue);
            tourHouseAdvisorInfoMapper.insertList(tourHouseAdvisorInfoList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseAdvisorInfo tourHouseAdvisorInfo, TourHouseAdvisorInfo originalTourHouseAdvisorInfo) {
        MyModelUtil.fillCommonsForUpdate(tourHouseAdvisorInfo, originalTourHouseAdvisorInfo);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseAdvisorInfo> uw = this.createUpdateQueryForNullValue(tourHouseAdvisorInfo, tourHouseAdvisorInfo.getId());
        return tourHouseAdvisorInfoMapper.update(tourHouseAdvisorInfo, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (tourHouseAdvisorInfoMapper.deleteById(id) == 0) {
            return false;
        }
        // 开始删除多对多父表的关联
        TourHouseAdvisorRelation tourHouseAdvisorRelation = new TourHouseAdvisorRelation();
        tourHouseAdvisorRelation.setAdvisorId(id);
        tourHouseAdvisorRelationMapper.delete(new QueryWrapper<>(tourHouseAdvisorRelation));
        return true;
    }

    @Override
    public List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoList(TourHouseAdvisorInfo filter, String orderBy) {
        return tourHouseAdvisorInfoMapper.getTourHouseAdvisorInfoList(filter, orderBy);
    }

    @Override
    public List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoListWithRelation(TourHouseAdvisorInfo filter, String orderBy) {
        List<TourHouseAdvisorInfo> resultList = tourHouseAdvisorInfoMapper.getTourHouseAdvisorInfoList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public List<TourHouseAdvisorInfo> getNotInTourHouseAdvisorInfoListByHouseId(Long houseId, TourHouseAdvisorInfo filter, String orderBy) {
        List<TourHouseAdvisorInfo> resultList;
        if (houseId != null) {
            resultList = tourHouseAdvisorInfoMapper.getNotInTourHouseAdvisorInfoListByHouseId(houseId, filter, orderBy);
        } else {
            resultList = getTourHouseAdvisorInfoList(filter, orderBy);
        }
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        return resultList;
    }

    @Override
    public List<TourHouseAdvisorInfo> getTourHouseAdvisorInfoListByHouseId(
            Long houseId, TourHouseAdvisorInfo filter, TourHouseAdvisorRelation tourHouseAdvisorRelationFilter, String orderBy) {
        List<TourHouseAdvisorInfo> resultList =
                tourHouseAdvisorInfoMapper.getTourHouseAdvisorInfoListByHouseId(houseId, filter, tourHouseAdvisorRelationFilter, orderBy);
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        // 这里是为中间表TourHouseAdvisorRelation中关联的静态字典，进行基于注解的数据绑定。
        List<TourHouseAdvisorRelation> tourHouseAdvisorRelationList = resultList.stream()
                .filter(c -> c.getTourHouseAdvisorRelation() != null)
                .map(TourHouseAdvisorInfo::getTourHouseAdvisorRelation).collect(Collectors.toList());
//        tourHouseAdvisorRelationService.buildRelationForDataList(
//                tourHouseAdvisorRelationList, MyRelationParam.dictOnly(), CollUtil.newHashSet("houseId", "advisorId"));
        return resultList;
    }

    @Override
    public Map<Long, List<TourHouseAdvisorInfo>> getTourHouseAdvisorInfoListMapByTourHouseSaleInfoList(
            List<TourHouseSaleInfo> tourHouseSaleInfoList, TourHouseAdvisorInfo filter, String orderBy) {
        List<Long> houseIds = tourHouseSaleInfoList.stream().map(TourHouseSaleInfo::getId).collect(Collectors.toList());
        List<TourHouseAdvisorInfo> resultList =
                tourHouseAdvisorInfoMapper.getTourHouseAdvisorInfoListByHouseIds(houseIds, filter, orderBy);
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        return resultList.stream().collect(Collectors.groupingBy(s -> s.getTourHouseAdvisorRelation().getHouseId()));
    }

    @Override
    public List<TourHouseAdvisorInfo> getAdvisors(Long id) {
        List<TourHouseAdvisorInfo> advisorInfos = tourHouseAdvisorInfoMapper.getAdvisors(id);
        return advisorInfos;
    }

    @Override
    public CallResult verifyRelatedData(TourHouseAdvisorInfo tourHouseAdvisorInfo, TourHouseAdvisorInfo originalTourHouseAdvisorInfo) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseAdvisorInfo, originalTourHouseAdvisorInfo, TourHouseAdvisorInfo::getDevelopersId)
                && !tourHouseDevelopersTypeService.existId(tourHouseAdvisorInfo.getDevelopersId())) {
            return CallResult.error(String.format(errorMessageFormat, "所属开发商"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseAdvisorInfo, originalTourHouseAdvisorInfo, TourHouseAdvisorInfo::getType)
                && !globalDictService.existDictItemFromCache("AdvisorType", tourHouseAdvisorInfo.getType())) {
            return CallResult.error(String.format(errorMessageFormat, "顾问类型 0.顾问 1.房东"));
        }
        return CallResult.ok();
    }

    private TourHouseAdvisorInfo buildDefaultValue(TourHouseAdvisorInfo tourHouseAdvisorInfo) {
        MyModelUtil.fillCommonsForInsert(tourHouseAdvisorInfo);
        tourHouseAdvisorInfo.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourHouseAdvisorInfo, "isGoldAdvisor", IsGoldAdvisor.HOUSE_NO);
        return tourHouseAdvisorInfo;
    }
}
