package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 买卖房与顾问的关联表数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourHouseAdvisorRelationService")
public class TourHouseAdvisorRelationServiceImpl extends BaseService<TourHouseAdvisorRelation, Long> implements TourHouseAdvisorRelationService {

    @Autowired
    private TourHouseAdvisorRelationMapper tourHouseAdvisorRelationMapper;
    @Autowired
    private TourHouseSaleInfoService tourHouseSaleInfoService;
    @Autowired
    private TourHouseAdvisorInfoService tourHouseAdvisorInfoService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseAdvisorRelation> mapper() {
        return tourHouseAdvisorRelationMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseAdvisorRelation saveNew(TourHouseAdvisorRelation tourHouseAdvisorRelation) {
        tourHouseAdvisorRelationMapper.insert(this.buildDefaultValue(tourHouseAdvisorRelation));
        return tourHouseAdvisorRelation;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseAdvisorRelation> tourHouseAdvisorRelationList) {
        if (CollUtil.isNotEmpty(tourHouseAdvisorRelationList)) {
            tourHouseAdvisorRelationList.forEach(this::buildDefaultValue);
            tourHouseAdvisorRelationMapper.insertList(tourHouseAdvisorRelationList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseAdvisorRelation saveNewWithRelation(TourHouseAdvisorRelation tourHouseAdvisorRelation, JSONObject relationData) {
        this.saveNew(tourHouseAdvisorRelation);
        this.saveOrUpdateOneToOneRelationData(tourHouseAdvisorRelation, relationData);
        return tourHouseAdvisorRelation;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseAdvisorRelation tourHouseAdvisorRelation, TourHouseAdvisorRelation originalTourHouseAdvisorRelation) {
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseAdvisorRelation> uw = this.createUpdateQueryForNullValue(tourHouseAdvisorRelation, tourHouseAdvisorRelation.getId());
        return tourHouseAdvisorRelationMapper.update(tourHouseAdvisorRelation, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourHouseAdvisorRelation tourHouseAdvisorRelation, TourHouseAdvisorRelation originalTourHouseAdvisorRelation, JSONObject relationData) {
        // tourHouseAdvisorRelation 为空的时候，无需修改主表数据。
        if (tourHouseAdvisorRelation != null && !this.update(tourHouseAdvisorRelation, originalTourHouseAdvisorRelation)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalTourHouseAdvisorRelation, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(TourHouseAdvisorRelation tourHouseAdvisorRelation, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        TourHouseSaleInfo tourHouseSaleInfo = relationData.getObject("tourHouseSaleInfo", TourHouseSaleInfo.class);
        if (tourHouseSaleInfo != null) {
            tourHouseSaleInfo.setId(tourHouseAdvisorRelation.getHouseId());
            tourHouseSaleInfoService.saveNewOrUpdate(tourHouseSaleInfo,
                    tourHouseSaleInfoService::saveNew, tourHouseSaleInfoService::update);
        }
        TourHouseAdvisorInfo tourHouseAdvisorInfo = relationData.getObject("tourHouseAdvisorInfo", TourHouseAdvisorInfo.class);
        if (tourHouseAdvisorInfo != null) {
            tourHouseAdvisorInfo.setId(tourHouseAdvisorRelation.getAdvisorId());
            tourHouseAdvisorInfoService.saveNewOrUpdate(tourHouseAdvisorInfo,
                    tourHouseAdvisorInfoService::saveNew, tourHouseAdvisorInfoService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        TourHouseAdvisorRelation tourHouseAdvisorRelation = tourHouseAdvisorRelationMapper.selectById(id);
        if (tourHouseAdvisorRelation == null) {
            return false;
        }
        if (tourHouseAdvisorRelationMapper.deleteById(id) == 0) {
            return false;
        }
        return true;
    }

    @Override
    public List<TourHouseAdvisorRelation> getTourHouseAdvisorRelationList(TourHouseAdvisorRelation filter, String orderBy) {
        return tourHouseAdvisorRelationMapper.getTourHouseAdvisorRelationList(filter, orderBy);
    }

    @Override
    public List<TourHouseAdvisorRelation> getTourHouseAdvisorRelationListWithRelation(TourHouseAdvisorRelation filter, String orderBy) {
        List<TourHouseAdvisorRelation> resultList = tourHouseAdvisorRelationMapper.getTourHouseAdvisorRelationList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourHouseAdvisorRelation buildDefaultValue(TourHouseAdvisorRelation tourHouseAdvisorRelation) {
        return tourHouseAdvisorRelation;
    }
}
