package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.app.website.vo.HouseMenuVo;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 租售房区域菜单管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
//@MyDataSource(DataSourceType.REAL_TEST)
@Service("tourHouseAreaMenuService")
public class TourHouseAreaMenuServiceImpl extends BaseService<TourHouseAreaMenu, Long> implements TourHouseAreaMenuService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourHouseAreaMenuMapper tourHouseAreaMenuMapper;
    @Autowired
    private TourHouseAreaMenuService tourHouseAreaMenuService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseAreaMenu> mapper() {
        return tourHouseAreaMenuMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseAreaMenu saveNew(TourHouseAreaMenu tourHouseAreaMenu) {
        tourHouseAreaMenuMapper.insert(this.buildDefaultValue(tourHouseAreaMenu));
        return tourHouseAreaMenu;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseAreaMenu> tourHouseAreaMenuList) {
        if (CollUtil.isNotEmpty(tourHouseAreaMenuList)) {
            tourHouseAreaMenuList.forEach(this::buildDefaultValue);
            tourHouseAreaMenuMapper.insertList(tourHouseAreaMenuList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseAreaMenu> tourHouseAreaMenuList, int batchSize) {
        if (CollUtil.isEmpty(tourHouseAreaMenuList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourHouseAreaMenuList.size(), start + batchSize);
            List<TourHouseAreaMenu> subList = tourHouseAreaMenuList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourHouseAreaMenuMapper.insertList(subList);
            if (end == tourHouseAreaMenuList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseAreaMenu tourHouseAreaMenu, TourHouseAreaMenu originalTourHouseAreaMenu) {
        MyModelUtil.fillCommonsForUpdate(tourHouseAreaMenu, originalTourHouseAreaMenu);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseAreaMenu> uw = this.createUpdateQueryForNullValue(tourHouseAreaMenu, tourHouseAreaMenu.getId());
        return tourHouseAreaMenuMapper.update(tourHouseAreaMenu, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourHouseAreaMenuMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourHouseAreaMenu> getTourHouseAreaMenuList(TourHouseAreaMenu filter, String orderBy) {
        return tourHouseAreaMenuMapper.getTourHouseAreaMenuList(filter, orderBy);
    }

    @Override
    public List<TourHouseAreaMenu> getTourHouseAreaMenuListWithRelation(TourHouseAreaMenu filter, String orderBy) {
        List<TourHouseAreaMenu> resultList = tourHouseAreaMenuMapper.getTourHouseAreaMenuList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public boolean hasChildren(Long id) {
        TourHouseAreaMenu filter = new TourHouseAreaMenu();
        filter.setParentId(id);
        return getCountByFilter(filter) > 0;
    }

    @Override
    public CallResult verifyImportList(List<TourHouseAreaMenu> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", TourHouseAreaMenu::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "isHotspot")) {
            callResult = verifyImportForConstDict(dataList, "isHotspotDictMap", TourHouseAreaMenu::getIsHotspot);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "levelNum")) {
            callResult = verifyImportForGlobalDict(dataList, "levelNumDictMap", TourHouseAreaMenu::getLevelNum);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "type")) {
            callResult = verifyImportForGlobalDict(dataList, "typeDictMap", TourHouseAreaMenu::getType);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "parentId")) {
            callResult = verifyImportForDatasourceDict(dataList, "parentIdDictMap", TourHouseAreaMenu::getParentId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourHouseAreaMenu tourHouseAreaMenu, TourHouseAreaMenu originalTourHouseAreaMenu) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseAreaMenu, originalTourHouseAreaMenu, TourHouseAreaMenu::getParentId)
                && !tourHouseAreaMenuService.existId(tourHouseAreaMenu.getParentId())) {
            return CallResult.error(String.format(errorMessageFormat, "父级id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseAreaMenu, originalTourHouseAreaMenu, TourHouseAreaMenu::getLevelNum)
                && !globalDictService.existDictItemFromCache("HouseAreaLevelNum", tourHouseAreaMenu.getLevelNum())) {
            return CallResult.error(String.format(errorMessageFormat, "菜单等级"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseAreaMenu, originalTourHouseAreaMenu, TourHouseAreaMenu::getType)
                && !globalDictService.existDictItemFromCache("IsSaleOrRent", tourHouseAreaMenu.getType())) {
            return CallResult.error(String.format(errorMessageFormat, "租售类型：0：出售 1：租赁"));
        }
        return CallResult.ok();
    }

    private TourHouseAreaMenu buildDefaultValue(TourHouseAreaMenu tourHouseAreaMenu) {
        if (tourHouseAreaMenu.getId() == null) {
            tourHouseAreaMenu.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourHouseAreaMenu);
        tourHouseAreaMenu.setDataState(GlobalDeletedFlag.NORMAL);
        return tourHouseAreaMenu;
    }

    //区域树
    @Override
    public List<HouseMenuVo> getHouseMenuTree() throws Exception {
        TourHouseAreaMenu filter = new TourHouseAreaMenu();
        filter.setEnable(1);
        filter.setDataState(1);
        List<TourHouseAreaMenu> allMenu = tourHouseAreaMenuService.getTourHouseAreaMenuList(filter, null);
        // 所有的确按照父级分组
        Map<Long, List<TourHouseAreaMenu>> collect = allMenu.stream().filter(e ->e.getParentId() != null).collect(Collectors.groupingBy(TourHouseAreaMenu::getParentId));
        // 获取所有一级地区
        List<TourHouseAreaMenu> list = allMenu.stream().filter(e -> e.getParentId() != null && e.getParentId() == 0l).toList();
        List<HouseMenuVo> resultList = MyModelUtil.copyCollectionTo(list, HouseMenuVo.class);
        this.findChildren(resultList, collect);
        return resultList;
    }

    /**
     * 递归填充子区域
     * @param resultList
     * @param collect
     */
    private void findChildren(List<HouseMenuVo> resultList, Map<Long, List<TourHouseAreaMenu>> collect){
        for (HouseMenuVo menuVo : resultList) {
            List<TourHouseAreaMenu> childrens = collect.get(Long.parseLong(menuVo.getId()));
            if(childrens != null && childrens.size() > 0){
                List<HouseMenuVo> childrenVos = MyModelUtil.copyCollectionTo(childrens, HouseMenuVo.class);
                menuVo.setChildren(childrenVos);
                this.findChildren(childrenVos, collect);
            }
        }
    }
}
