package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 租赁房屋配套设施数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourHouseEquipmentService")
public class TourHouseEquipmentServiceImpl extends BaseService<TourHouseEquipment, Long> implements TourHouseEquipmentService {

    @Autowired
    private TourHouseEquipmentMapper tourHouseEquipmentMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseEquipment> mapper() {
        return tourHouseEquipmentMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseEquipment saveNew(TourHouseEquipment tourHouseEquipment) {
        tourHouseEquipmentMapper.insert(this.buildDefaultValue(tourHouseEquipment));
        return tourHouseEquipment;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseEquipment> tourHouseEquipmentList) {
        if (CollUtil.isNotEmpty(tourHouseEquipmentList)) {
            tourHouseEquipmentList.forEach(this::buildDefaultValue);
            tourHouseEquipmentMapper.insertList(tourHouseEquipmentList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseEquipment tourHouseEquipment, TourHouseEquipment originalTourHouseEquipment) {
        MyModelUtil.fillCommonsForUpdate(tourHouseEquipment, originalTourHouseEquipment);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseEquipment> uw = this.createUpdateQueryForNullValue(tourHouseEquipment, tourHouseEquipment.getId());
        return tourHouseEquipmentMapper.update(tourHouseEquipment, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourHouseEquipmentMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByHouseId(Long houseId) {
        TourHouseEquipment deletedObject = new TourHouseEquipment();
        deletedObject.setHouseId(houseId);
        return tourHouseEquipmentMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<TourHouseEquipment> getTourHouseEquipmentList(TourHouseEquipment filter, String orderBy) {
        return tourHouseEquipmentMapper.getTourHouseEquipmentList(filter, orderBy);
    }

    @Override
    public List<TourHouseEquipment> getTourHouseEquipmentListWithRelation(TourHouseEquipment filter, String orderBy) {
        List<TourHouseEquipment> resultList = tourHouseEquipmentMapper.getTourHouseEquipmentList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourHouseEquipment buildDefaultValue(TourHouseEquipment tourHouseEquipment) {
        MyModelUtil.fillCommonsForInsert(tourHouseEquipment);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"icebox",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"washer",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"calorifier",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"broadband",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"sofa",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"rangeHood",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"gasStove",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"ableCook",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"television",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"airConditioner",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"wardrobe",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"bed",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"restRoom",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"smartDoorLock",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"balcony",0);
        MyModelUtil.setDefaultValue(tourHouseEquipment,"heating",0);
        tourHouseEquipment.setDataState(GlobalDeletedFlag.NORMAL);
        return tourHouseEquipment;
    }
}
