package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 房屋图集信息管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
//@MyDataSource(DataSourceType.REAL_TEST)
@Service("tourHouseFilesService")
public class TourHouseFilesServiceImpl extends BaseService<TourHouseFiles, Long> implements TourHouseFilesService {

    @Autowired
    private TourHouseFilesMapper tourHouseFilesMapper;
    @Autowired
    private TourHouseSaleInfoService tourHouseSaleInfoService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseFiles> mapper() {
        return tourHouseFilesMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseFiles saveNew(TourHouseFiles tourHouseFiles) {
        tourHouseFilesMapper.insert(this.buildDefaultValue(tourHouseFiles));
        return tourHouseFiles;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseFiles> tourHouseFilesList) {
        if (CollUtil.isNotEmpty(tourHouseFilesList)) {
            tourHouseFilesList.forEach(this::buildDefaultValue);
            tourHouseFilesMapper.insertList(tourHouseFilesList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseFiles tourHouseFiles, TourHouseFiles originalTourHouseFiles) {
        MyModelUtil.fillCommonsForUpdate(tourHouseFiles, originalTourHouseFiles);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseFiles> uw = this.createUpdateQueryForNullValue(tourHouseFiles, tourHouseFiles.getId());
        return tourHouseFilesMapper.update(tourHouseFiles, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourHouseFilesMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByTourHouseId(Long tourHouseId) {
        TourHouseFiles deletedObject = new TourHouseFiles();
        deletedObject.setTourHouseId(tourHouseId);
        return tourHouseFilesMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateBatchByTourHouseId(Long tourHouseId, List<TourHouseFiles> dataList) {
        this.updateBatchOneToManyRelation("tourHouseId", tourHouseId,
                null, null, dataList, this::saveNewBatch);
    }

    @Override
    public List<TourHouseFiles> getTourHouseFilesList(TourHouseFiles filter, String orderBy) {
        return tourHouseFilesMapper.getTourHouseFilesList(filter, orderBy);
    }

    @Override
    public List<TourHouseFiles> getTourHouseFilesListWithRelation(TourHouseFiles filter, String orderBy) {
        List<TourHouseFiles> resultList = tourHouseFilesMapper.getTourHouseFilesList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourHouseFiles tourHouseFiles, TourHouseFiles originalTourHouseFiles) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseFiles, originalTourHouseFiles, TourHouseFiles::getFileType)
                && !globalDictService.existDictItemFromCache("HouseFileType", tourHouseFiles.getFileType())) {
            return CallResult.error(String.format(errorMessageFormat, "文件类型 0房屋图集  1户型图集 2周边配套图集 3小区介绍图集"));
        }
        //这里是一对多的验证
        if (this.needToVerify(tourHouseFiles, originalTourHouseFiles, TourHouseFiles::getTourHouseId)
                && !tourHouseSaleInfoService.existId(tourHouseFiles.getTourHouseId())) {
            return CallResult.error(String.format(errorMessageFormat, "房屋表id"));
        }
        return CallResult.ok();
    }

    private TourHouseFiles buildDefaultValue(TourHouseFiles tourHouseFiles) {
        MyModelUtil.fillCommonsForInsert(tourHouseFiles);
        tourHouseFiles.setDataState(GlobalDeletedFlag.NORMAL);
        return tourHouseFiles;
    }

}
