package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 租赁房屋信息数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourHouseRentInfoService")
public class TourHouseRentInfoServiceImpl extends BaseService<TourHouseRentInfo, Long> implements TourHouseRentInfoService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourHouseRentInfoMapper tourHouseRentInfoMapper;
    @Autowired
    private TourHouseAreaMenuService tourHouseAreaMenuService;
    @Autowired
    private TourHouseEquipmentService tourHouseEquipmentService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourHouseRentInfo> mapper() {
        return tourHouseRentInfoMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseRentInfo saveNew(TourHouseRentInfo tourHouseRentInfo) {
        tourHouseRentInfoMapper.insert(this.buildDefaultValue(tourHouseRentInfo));
        return tourHouseRentInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourHouseRentInfo> tourHouseRentInfoList) {
        if (CollUtil.isNotEmpty(tourHouseRentInfoList)) {
            tourHouseRentInfoList.forEach(this::buildDefaultValue);
            tourHouseRentInfoMapper.insertList(tourHouseRentInfoList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourHouseRentInfo saveNewWithRelation(TourHouseRentInfo tourHouseRentInfo, JSONObject relationData) {
        this.saveNew(tourHouseRentInfo);
        this.saveOrUpdateOneToOneRelationData(tourHouseRentInfo, relationData);
        return tourHouseRentInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourHouseRentInfo tourHouseRentInfo, TourHouseRentInfo originalTourHouseRentInfo) {
        MyModelUtil.fillCommonsForUpdate(tourHouseRentInfo, originalTourHouseRentInfo);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourHouseRentInfo> uw = this.createUpdateQueryForNullValue(tourHouseRentInfo, tourHouseRentInfo.getId());
        return tourHouseRentInfoMapper.update(tourHouseRentInfo, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourHouseRentInfo tourHouseRentInfo, TourHouseRentInfo originalTourHouseRentInfo, JSONObject relationData) {
        // tourHouseRentInfo 为空的时候，无需修改主表数据。
        if (tourHouseRentInfo != null && !this.update(tourHouseRentInfo, originalTourHouseRentInfo)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalTourHouseRentInfo, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(TourHouseRentInfo tourHouseRentInfo, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        TourHouseEquipment tourHouseEquipment = relationData.getObject("tourHouseEquipment", TourHouseEquipment.class);
        if (tourHouseEquipment != null) {
            tourHouseEquipment.setHouseId(tourHouseRentInfo.getId());
            tourHouseEquipmentService.saveNewOrUpdate(tourHouseEquipment,
                    tourHouseEquipmentService::saveNew, tourHouseEquipmentService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (tourHouseRentInfoMapper.deleteById(id) == 0) {
            return false;
        }
        tourHouseEquipmentService.removeByHouseId(id);
        return true;
    }

    @Override
    public List<TourHouseRentInfo> getTourHouseRentInfoList(TourHouseRentInfo filter, String orderBy) {
        return tourHouseRentInfoMapper.getTourHouseRentInfoList(filter, orderBy);
    }

    @Override
    public List<TourHouseRentInfo> getTourHouseRentInfoListWithRelation(TourHouseRentInfo filter, TourHouseEquipment tourHouseEquipmentFilter, String orderBy) {
        List<TourHouseRentInfo> resultList =
                tourHouseRentInfoMapper.getTourHouseRentInfoListEx(filter, tourHouseEquipmentFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourHouseRentInfo tourHouseRentInfo, TourHouseRentInfo originalTourHouseRentInfo) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getAreaId)
                && !tourHouseAreaMenuService.existId(tourHouseRentInfo.getAreaId())) {
            return CallResult.error(String.format(errorMessageFormat, "所属地区类型"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getRentType)
                && !globalDictService.existDictItemFromCache("HouseRentType", tourHouseRentInfo.getRentType())) {
            return CallResult.error(String.format(errorMessageFormat, "租赁方式 0整租 1合租 2个人房源 3短租"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getOfficeType)
                && !globalDictService.existDictItemFromCache("OfficeType", tourHouseRentInfo.getOfficeType())) {
            return CallResult.error(String.format(errorMessageFormat, "厅室类型"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getHouseType)
                && !globalDictService.existDictItemFromCache("HouseType", tourHouseRentInfo.getHouseType())) {
            return CallResult.error(String.format(errorMessageFormat, "房屋类型 别墅、公寓..."));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getRentKind)
                && !globalDictService.existDictItemFromCache("RentKind", tourHouseRentInfo.getRentKind())) {
            return CallResult.error(String.format(errorMessageFormat, "租金类型(0无租金;1押一付一;2押一付二;3押一付三....)"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getOrientation)
                && !globalDictService.existDictItemFromCache("Orientation", tourHouseRentInfo.getOrientation())) {
            return CallResult.error(String.format(errorMessageFormat, "朝向类型"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourHouseRentInfo, originalTourHouseRentInfo, TourHouseRentInfo::getDecoration)
                && !globalDictService.existDictItemFromCache("Decoration", tourHouseRentInfo.getDecoration())) {
            return CallResult.error(String.format(errorMessageFormat, "装修类型"));
        }
        return CallResult.ok();
    }

    private TourHouseRentInfo buildDefaultValue(TourHouseRentInfo tourHouseRentInfo) {
        if (tourHouseRentInfo.getId() == null) {
            tourHouseRentInfo.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourHouseRentInfo);
        tourHouseRentInfo.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourHouseRentInfo, "remark", "");
        return tourHouseRentInfo;
    }


    @Override
    public List<TourHouseRentInfo> getTourHouseRentInfoListWithArea(TourHouseRentInfo filter, String orderBy) {
        List<TourHouseRentInfo> resultList = tourHouseRentInfoMapper.getTourHouseRentInfoListWithArea(filter, orderBy);

        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }
}
