package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 消息通知表数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourNoticeService")
public class TourNoticeServiceImpl extends BaseService<TourNotice, Long> implements TourNoticeService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourNoticeMapper tourNoticeMapper;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourNotice> mapper() {
        return tourNoticeMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourNotice saveNew(TourNotice tourNotice) {
        tourNoticeMapper.insert(this.buildDefaultValue(tourNotice));
        return tourNotice;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourNotice> tourNoticeList) {
        if (CollUtil.isNotEmpty(tourNoticeList)) {
            tourNoticeList.forEach(this::buildDefaultValue);
            tourNoticeMapper.insertList(tourNoticeList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourNotice tourNotice, TourNotice originalTourNotice) {
        MyModelUtil.fillCommonsForUpdate(tourNotice, originalTourNotice);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourNotice> uw = this.createUpdateQueryForNullValue(tourNotice, tourNotice.getId());
        return tourNoticeMapper.update(tourNotice, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourNoticeMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourNotice> getTourNoticeList(TourNotice filter, String orderBy) {
        return tourNoticeMapper.getTourNoticeList(filter, orderBy);
    }

    @Override
    public List<TourNotice> getTourNoticeListWithRelation(TourNotice filter, String orderBy) {
        List<TourNotice> resultList = tourNoticeMapper.getTourNoticeList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourNotice tourNotice, TourNotice originalTourNotice) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourNotice, originalTourNotice, TourNotice::getNoticeType)
                && !globalDictService.existDictItemFromCache("NoticeType", tourNotice.getNoticeType())) {
            return CallResult.error(String.format(errorMessageFormat, "消息类型：1消息回复；2收藏提醒；3游记审核"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourNotice, originalTourNotice, TourNotice::getBeenRead)
                && !globalDictService.existDictItemFromCache("BeenReed", tourNotice.getBeenRead())) {
            return CallResult.error(String.format(errorMessageFormat, "是否已读：0未读；1已读"));
        }
        return CallResult.ok();
    }

    private TourNotice buildDefaultValue(TourNotice tourNotice) {
        if (tourNotice.getId() == null) {
            tourNotice.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourNotice);
        tourNotice.setDataState(GlobalDeletedFlag.NORMAL);
        return tourNotice;
    }
}
