package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 旅游订单游客数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourOrderPassenageService")
public class TourOrderPassenageServiceImpl extends BaseService<TourOrderPassenage, Long> implements TourOrderPassenageService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourOrderPassenageMapper tourOrderPassenageMapper;
    @Autowired
    private TourOrderService tourOrderService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourOrderPassenage> mapper() {
        return tourOrderPassenageMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourOrderPassenage saveNew(TourOrderPassenage tourOrderPassenage) {
        tourOrderPassenageMapper.insert(this.buildDefaultValue(tourOrderPassenage));
        return tourOrderPassenage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourOrderPassenage> tourOrderPassenageList) {
        if (CollUtil.isNotEmpty(tourOrderPassenageList)) {
            tourOrderPassenageList.forEach(this::buildDefaultValue);
            tourOrderPassenageMapper.insertList(tourOrderPassenageList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourOrderPassenage tourOrderPassenage, TourOrderPassenage originalTourOrderPassenage) {
        MyModelUtil.fillCommonsForUpdate(tourOrderPassenage, originalTourOrderPassenage);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourOrderPassenage> uw = this.createUpdateQueryForNullValue(tourOrderPassenage, tourOrderPassenage.getId());
        return tourOrderPassenageMapper.update(tourOrderPassenage, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourOrderPassenageMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByOrderId(Long orderId) {
        TourOrderPassenage deletedObject = new TourOrderPassenage();
        deletedObject.setOrderId(orderId);
        return tourOrderPassenageMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Override
    public List<TourOrderPassenage> getTourOrderPassenageList(TourOrderPassenage filter, String orderBy) {
        return tourOrderPassenageMapper.getTourOrderPassenageList(filter, orderBy);
    }

    @Override
    public List<TourOrderPassenage> getTourOrderPassenageListWithRelation(TourOrderPassenage filter, String orderBy) {
        List<TourOrderPassenage> resultList = tourOrderPassenageMapper.getTourOrderPassenageList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourOrderPassenage tourOrderPassenage, TourOrderPassenage originalTourOrderPassenage) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是一对多的验证
        if (this.needToVerify(tourOrderPassenage, originalTourOrderPassenage, TourOrderPassenage::getOrderId)
                && !tourOrderService.existId(tourOrderPassenage.getOrderId())) {
            return CallResult.error(String.format(errorMessageFormat, "订单号信息"));
        }
        return CallResult.ok();
    }

    private TourOrderPassenage buildDefaultValue(TourOrderPassenage tourOrderPassenage) {
        if (tourOrderPassenage.getId() == null) {
            tourOrderPassenage.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourOrderPassenage);
        tourOrderPassenage.setDataState(GlobalDeletedFlag.NORMAL);
        return tourOrderPassenage;
    }
}
