package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 开团申请列表数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourProjectGroupPurchaseApplyService")
public class TourProjectGroupPurchaseApplyServiceImpl extends BaseService<TourProjectGroupPurchaseApply, Long> implements TourProjectGroupPurchaseApplyService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourProjectGroupPurchaseApplyMapper tourProjectGroupPurchaseApplyMapper;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourProjectGroupPurchaseApply> mapper() {
        return tourProjectGroupPurchaseApplyMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourProjectGroupPurchaseApply saveNew(TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply) {
        tourProjectGroupPurchaseApplyMapper.insert(this.buildDefaultValue(tourProjectGroupPurchaseApply));
        return tourProjectGroupPurchaseApply;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourProjectGroupPurchaseApply> tourProjectGroupPurchaseApplyList) {
        if (CollUtil.isNotEmpty(tourProjectGroupPurchaseApplyList)) {
            tourProjectGroupPurchaseApplyList.forEach(this::buildDefaultValue);
            tourProjectGroupPurchaseApplyMapper.insertList(tourProjectGroupPurchaseApplyList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply, TourProjectGroupPurchaseApply originalTourProjectGroupPurchaseApply) {
        MyModelUtil.fillCommonsForUpdate(tourProjectGroupPurchaseApply, originalTourProjectGroupPurchaseApply);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourProjectGroupPurchaseApply> uw = this.createUpdateQueryForNullValue(tourProjectGroupPurchaseApply, tourProjectGroupPurchaseApply.getId());
        return tourProjectGroupPurchaseApplyMapper.update(tourProjectGroupPurchaseApply, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourProjectGroupPurchaseApplyMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourProjectGroupPurchaseApply> getTourProjectGroupPurchaseApplyList(TourProjectGroupPurchaseApply filter, String orderBy) {
        return tourProjectGroupPurchaseApplyMapper.getTourProjectGroupPurchaseApplyList(filter, orderBy);
    }

    @Override
    public List<TourProjectGroupPurchaseApply> getTourProjectGroupPurchaseApplyListWithRelation(TourProjectGroupPurchaseApply filter, String orderBy) {
        List<TourProjectGroupPurchaseApply> resultList = tourProjectGroupPurchaseApplyMapper.getTourProjectGroupPurchaseApplyList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply, TourProjectGroupPurchaseApply originalTourProjectGroupPurchaseApply) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourProjectGroupPurchaseApply, originalTourProjectGroupPurchaseApply, TourProjectGroupPurchaseApply::getIsHandle)
                && !globalDictService.existDictItemFromCache("IsHandle", tourProjectGroupPurchaseApply.getIsHandle())) {
            return CallResult.error(String.format(errorMessageFormat, "是否处理(0:未处理 1:已处理)"));
        }
        return CallResult.ok();
    }

    private TourProjectGroupPurchaseApply buildDefaultValue(TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply) {
        if (tourProjectGroupPurchaseApply.getId() == null) {
            tourProjectGroupPurchaseApply.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourProjectGroupPurchaseApply);
        tourProjectGroupPurchaseApply.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourProjectGroupPurchaseApply, "remark", "");
        return tourProjectGroupPurchaseApply;
    }
}
