package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 拼团返利记录数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourProjectGroupPurchaseRebateService")
public class TourProjectGroupPurchaseRebateServiceImpl extends BaseService<TourProjectGroupPurchaseRebate, Long> implements TourProjectGroupPurchaseRebateService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourProjectGroupPurchaseRebateMapper tourProjectGroupPurchaseRebateMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourProjectGroupPurchaseRebate> mapper() {
        return tourProjectGroupPurchaseRebateMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourProjectGroupPurchaseRebate saveNew(TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate) {
        tourProjectGroupPurchaseRebateMapper.insert(this.buildDefaultValue(tourProjectGroupPurchaseRebate));
        return tourProjectGroupPurchaseRebate;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourProjectGroupPurchaseRebate> tourProjectGroupPurchaseRebateList) {
        if (CollUtil.isNotEmpty(tourProjectGroupPurchaseRebateList)) {
            tourProjectGroupPurchaseRebateList.forEach(this::buildDefaultValue);
            tourProjectGroupPurchaseRebateMapper.insertList(tourProjectGroupPurchaseRebateList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate, TourProjectGroupPurchaseRebate originalTourProjectGroupPurchaseRebate) {
        MyModelUtil.fillCommonsForUpdate(tourProjectGroupPurchaseRebate, originalTourProjectGroupPurchaseRebate);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourProjectGroupPurchaseRebate> uw = this.createUpdateQueryForNullValue(tourProjectGroupPurchaseRebate, tourProjectGroupPurchaseRebate.getId());
        return tourProjectGroupPurchaseRebateMapper.update(tourProjectGroupPurchaseRebate, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourProjectGroupPurchaseRebateMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourProjectGroupPurchaseRebate> getTourProjectGroupPurchaseRebateList(TourProjectGroupPurchaseRebate filter, String orderBy) {
        return tourProjectGroupPurchaseRebateMapper.getTourProjectGroupPurchaseRebateList(filter, orderBy);
    }

    @Override
    public List<TourProjectGroupPurchaseRebate> getTourProjectGroupPurchaseRebateListWithRelation(TourProjectGroupPurchaseRebate filter, String orderBy) {
        List<TourProjectGroupPurchaseRebate> resultList = tourProjectGroupPurchaseRebateMapper.getTourProjectGroupPurchaseRebateList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourProjectGroupPurchaseRebate buildDefaultValue(TourProjectGroupPurchaseRebate tourProjectGroupPurchaseRebate) {
        if (tourProjectGroupPurchaseRebate.getId() == null) {
            tourProjectGroupPurchaseRebate.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourProjectGroupPurchaseRebate);
        tourProjectGroupPurchaseRebate.setDataState(GlobalDeletedFlag.NORMAL);
        return tourProjectGroupPurchaseRebate;
    }
}
