package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.model.constant.VisaIndexFlag;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 签证顾问信息数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourVisaCounselorService")
public class TourVisaCounselorServiceImpl extends BaseService<TourVisaCounselor, Long> implements TourVisaCounselorService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourVisaCounselorMapper tourVisaCounselorMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourVisaCounselor> mapper() {
        return tourVisaCounselorMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourVisaCounselor saveNew(TourVisaCounselor tourVisaCounselor) {
        tourVisaCounselorMapper.insert(this.buildDefaultValue(tourVisaCounselor));
        return tourVisaCounselor;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaCounselor> tourVisaCounselorList) {
        if (CollUtil.isNotEmpty(tourVisaCounselorList)) {
            tourVisaCounselorList.forEach(this::buildDefaultValue);
            tourVisaCounselorMapper.insertList(tourVisaCounselorList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaCounselor> tourVisaCounselorList, int batchSize) {
        if (CollUtil.isEmpty(tourVisaCounselorList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourVisaCounselorList.size(), start + batchSize);
            List<TourVisaCounselor> subList = tourVisaCounselorList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourVisaCounselorMapper.insertList(subList);
            if (end == tourVisaCounselorList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourVisaCounselor tourVisaCounselor, TourVisaCounselor originalTourVisaCounselor) {
        MyModelUtil.fillCommonsForUpdate(tourVisaCounselor, originalTourVisaCounselor);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourVisaCounselor> uw = this.createUpdateQueryForNullValue(tourVisaCounselor, tourVisaCounselor.getId());
        return tourVisaCounselorMapper.update(tourVisaCounselor, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourVisaCounselorMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourVisaCounselor> getTourVisaCounselorList(TourVisaCounselor filter, String orderBy) {
        return tourVisaCounselorMapper.getTourVisaCounselorList(filter, orderBy);
    }

    @Override
    public List<TourVisaCounselor> getTourVisaCounselorListWithRelation(TourVisaCounselor filter, String orderBy) {
        List<TourVisaCounselor> resultList = tourVisaCounselorMapper.getTourVisaCounselorList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourVisaCounselor> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "level")) {
            callResult = verifyImportForConstDict(dataList, "levelDictMap", TourVisaCounselor::getLevel);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "indexFlag")) {
            callResult = verifyImportForConstDict(dataList, "indexFlagDictMap", TourVisaCounselor::getIndexFlag);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    private TourVisaCounselor buildDefaultValue(TourVisaCounselor tourVisaCounselor) {
        if (tourVisaCounselor.getId() == null) {
            tourVisaCounselor.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourVisaCounselor);
        tourVisaCounselor.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourVisaCounselor, "counselorAvatar", "");
        MyModelUtil.setDefaultValue(tourVisaCounselor, "counselorPhone", "");
        MyModelUtil.setDefaultValue(tourVisaCounselor, "counselorWechat", "");
        MyModelUtil.setDefaultValue(tourVisaCounselor, "counselorName", "");
        MyModelUtil.setDefaultValue(tourVisaCounselor, "indexFlag", VisaIndexFlag.VISAINDEXFLAG_NO);
        return tourVisaCounselor;
    }
}
