package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.model.constant.VisaOrderState;
import com.tourism.webadmin.back.model.constant.VisaPayState;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 签证订单类型信息数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourVisaOrderService")
public class TourVisaOrderServiceImpl extends BaseService<TourVisaOrder, Long> implements TourVisaOrderService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourVisaOrderMapper tourVisaOrderMapper;
    @Autowired
    private TourVisaService tourVisaService;
    @Autowired
    private TourVisaCounselorService tourVisaCounselorService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourVisaOrder> mapper() {
        return tourVisaOrderMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourVisaOrder saveNew(TourVisaOrder tourVisaOrder) {
        tourVisaOrderMapper.insert(this.buildDefaultValue(tourVisaOrder));
        return tourVisaOrder;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaOrder> tourVisaOrderList) {
        if (CollUtil.isNotEmpty(tourVisaOrderList)) {
            tourVisaOrderList.forEach(this::buildDefaultValue);
            tourVisaOrderMapper.insertList(tourVisaOrderList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaOrder> tourVisaOrderList, int batchSize) {
        if (CollUtil.isEmpty(tourVisaOrderList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourVisaOrderList.size(), start + batchSize);
            List<TourVisaOrder> subList = tourVisaOrderList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourVisaOrderMapper.insertList(subList);
            if (end == tourVisaOrderList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourVisaOrder tourVisaOrder, TourVisaOrder originalTourVisaOrder) {
        MyModelUtil.fillCommonsForUpdate(tourVisaOrder, originalTourVisaOrder);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourVisaOrder> uw = this.createUpdateQueryForNullValue(tourVisaOrder, tourVisaOrder.getId());
        return tourVisaOrderMapper.update(tourVisaOrder, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourVisaOrderMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourVisaOrder> getTourVisaOrderList(TourVisaOrder filter, String orderBy) {
        return tourVisaOrderMapper.getTourVisaOrderList(filter, orderBy);
    }

    @Override
    public List<TourVisaOrder> getTourVisaOrderListWithRelation(TourVisaOrder filter, String orderBy) {
        List<TourVisaOrder> resultList = tourVisaOrderMapper.getTourVisaOrderList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourVisaOrder> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "state")) {
            callResult = verifyImportForConstDict(dataList, "stateDictMap", TourVisaOrder::getState);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "payState")) {
            callResult = verifyImportForConstDict(dataList, "payStateDictMap", TourVisaOrder::getPayState);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "visaId")) {
            callResult = verifyImportForDatasourceDict(dataList, "visaIdDictMap", TourVisaOrder::getVisaId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "counselorId")) {
            callResult = verifyImportForDatasourceDict(dataList, "counselorIdDictMap", TourVisaOrder::getCounselorId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourVisaOrder tourVisaOrder, TourVisaOrder originalTourVisaOrder) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisaOrder, originalTourVisaOrder, TourVisaOrder::getVisaId)
                && !tourVisaService.existId(tourVisaOrder.getVisaId())) {
            return CallResult.error(String.format(errorMessageFormat, "签证商品id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisaOrder, originalTourVisaOrder, TourVisaOrder::getCounselorId)
                && !tourVisaCounselorService.existId(tourVisaOrder.getCounselorId())) {
            return CallResult.error(String.format(errorMessageFormat, "负责顾问"));
        }
        return CallResult.ok();
    }

    private TourVisaOrder buildDefaultValue(TourVisaOrder tourVisaOrder) {
        if (tourVisaOrder.getId() == null) {
            tourVisaOrder.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourVisaOrder);
        tourVisaOrder.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourVisaOrder, "orderName", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "orderPhone", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "orderIdCard", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "orderPrice", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "orderAddress", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "servicePrice", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "price", "");
        MyModelUtil.setDefaultValue(tourVisaOrder, "state", VisaOrderState.VISAORDERSTATE_CREATE);
        MyModelUtil.setDefaultValue(tourVisaOrder, "payState", VisaPayState.VISAPAY_NO);
        return tourVisaOrder;
    }
}
