package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.dao.TourVisaTypeMapper;
import com.tourism.webadmin.back.model.TourVisaType;
import com.tourism.webadmin.back.service.TourVisaTypeService;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 签证类型信息数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourVisaTypeService")
public class TourVisaTypeServiceImpl extends BaseService<TourVisaType, Long> implements TourVisaTypeService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourVisaTypeMapper tourVisaTypeMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourVisaType> mapper() {
        return tourVisaTypeMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourVisaType saveNew(TourVisaType tourVisaType) {
        tourVisaTypeMapper.insert(this.buildDefaultValue(tourVisaType));
        return tourVisaType;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaType> tourVisaTypeList) {
        if (CollUtil.isNotEmpty(tourVisaTypeList)) {
            tourVisaTypeList.forEach(this::buildDefaultValue);
            tourVisaTypeMapper.insertList(tourVisaTypeList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisaType> tourVisaTypeList, int batchSize) {
        if (CollUtil.isEmpty(tourVisaTypeList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourVisaTypeList.size(), start + batchSize);
            List<TourVisaType> subList = tourVisaTypeList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourVisaTypeMapper.insertList(subList);
            if (end == tourVisaTypeList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourVisaType tourVisaType, TourVisaType originalTourVisaType) {
        MyModelUtil.fillCommonsForUpdate(tourVisaType, originalTourVisaType);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourVisaType> uw = this.createUpdateQueryForNullValue(tourVisaType, tourVisaType.getId());
        return tourVisaTypeMapper.update(tourVisaType, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourVisaTypeMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourVisaType> getTourVisaTypeList(TourVisaType filter, String orderBy) {
        return tourVisaTypeMapper.getTourVisaTypeList(filter, orderBy);
    }

    @Override
    public List<TourVisaType> getTourVisaTypeListWithRelation(TourVisaType filter, String orderBy) {
        List<TourVisaType> resultList = tourVisaTypeMapper.getTourVisaTypeList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private TourVisaType buildDefaultValue(TourVisaType tourVisaType) {
        if (tourVisaType.getId() == null) {
            tourVisaType.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourVisaType);
        tourVisaType.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourVisaType, "title", "");
        return tourVisaType;
    }
}
