package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 旅游项目日历价格数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourismDatePriceService")
public class TourismDatePriceServiceImpl extends BaseService<TourismDatePrice, String> implements TourismDatePriceService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourismDatePriceMapper tourismDatePriceMapper;
    @Autowired
    private TourismProjectService tourismProjectService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourismDatePrice> mapper() {
        return tourismDatePriceMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourismDatePrice saveNew(TourismDatePrice tourismDatePrice) {
        tourismDatePriceMapper.insert(this.buildDefaultValue(tourismDatePrice));
        return tourismDatePrice;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourismDatePrice> tourismDatePriceList) {
        if (CollUtil.isNotEmpty(tourismDatePriceList)) {
            tourismDatePriceList.forEach(this::buildDefaultValue);
            tourismDatePriceMapper.insertList(tourismDatePriceList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourismDatePrice tourismDatePrice, TourismDatePrice originalTourismDatePrice) {
        MyModelUtil.fillCommonsForUpdate(tourismDatePrice, originalTourismDatePrice);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourismDatePrice> uw = this.createUpdateQueryForNullValue(tourismDatePrice, tourismDatePrice.getId());
        return tourismDatePriceMapper.update(tourismDatePrice, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourismDatePriceMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourismDatePrice> getTourismDatePriceList(TourismDatePrice filter, String orderBy) {
        return tourismDatePriceMapper.getTourismDatePriceList(filter, orderBy);
    }

    @Override
    public List<TourismDatePrice> getTourismDatePriceListWithRelation(TourismDatePrice filter, String orderBy) {
        List<TourismDatePrice> resultList = tourismDatePriceMapper.getTourismDatePriceList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourismDatePrice tourismDatePrice, TourismDatePrice originalTourismDatePrice) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourismDatePrice, originalTourismDatePrice, TourismDatePrice::getProjectId)
                && !tourismProjectService.existId(tourismDatePrice.getProjectId())) {
            return CallResult.error(String.format(errorMessageFormat, "旅游项目信息"));
        }
        return CallResult.ok();
    }

    @Override
    public TourismDatePrice getOneTourismDatePrice(TourismDatePrice tourismDatePrice) {
        LambdaQueryWrapper<TourismDatePrice> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TourismDatePrice::getProjectId, tourismDatePrice.getProjectId());
        queryWrapper.eq(TourismDatePrice::getDepartureDate, tourismDatePrice.getDepartureDate());
        TourismDatePrice one = tourismDatePriceMapper.selectOne(queryWrapper);
        if (one != null) {
            return one;
        }
        return null;
    }

    private TourismDatePrice buildDefaultValue(TourismDatePrice tourismDatePrice) {
        if (tourismDatePrice.getId() == null) {
            tourismDatePrice.setId(idGenerator.nextStringId());
        }
        MyModelUtil.fillCommonsForInsert(tourismDatePrice);
        tourismDatePrice.setDataState(GlobalDeletedFlag.NORMAL);
        return tourismDatePrice;
    }
}
