package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.common.core.upload.*;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.webadmin.back.dto.MultipartFileDto;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import com.tourism.common.additional.config.ApplicationConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 旅游项目管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourismProjectService")
public class TourismProjectServiceImpl extends BaseService<TourismProject, String> implements TourismProjectService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourismProjectMapper tourismProjectMapper;
    @Autowired
    private TourTravelNotesProjectRelationMapper tourTravelNotesProjectRelationMapper;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private TourismFileService tourismFileService;
    @Autowired
    private TourismContentService tourismContentService;
    @Autowired
    private TourTourismProjectTravelNotesService tourTourismProjectTravelNotesService;
    @Autowired
    private TourTravelNotesProjectRelationService tourTravelNotesProjectRelationService;
    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourismProject> mapper() {
        return tourismProjectMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourismProject saveNew(TourismProject tourismProject) {
        tourismProjectMapper.insert(this.buildDefaultValue(tourismProject));
        return tourismProject;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourismProject> tourismProjectList) {
        if (CollUtil.isNotEmpty(tourismProjectList)) {
            tourismProjectList.forEach(this::buildDefaultValue);
            tourismProjectMapper.insertList(tourismProjectList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourismProject> tourismProjectList, int batchSize) {
        if (CollUtil.isEmpty(tourismProjectList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourismProjectList.size(), start + batchSize);
            List<TourismProject> subList = tourismProjectList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourismProjectMapper.insertList(subList);
            if (end == tourismProjectList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourismProject saveNewWithRelation(TourismProject tourismProject, JSONObject relationData) throws Exception {
        this.saveNew(tourismProject);
        this.saveOrUpdateOneToOneRelationData(tourismProject, relationData);
        return tourismProject;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourismProject tourismProject, TourismProject originalTourismProject) {
//        MyModelUtil.fillCommonsForUpdate(tourismProject, originalTourismProject);
//        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
//        UpdateWrapper<TourismProject> uw = this.createUpdateQueryForNullValue(tourismProject, tourismProject.getId());
//        return tourismProjectMapper.update(tourismProject, uw) == 1;
        return tourismProjectMapper.updateById(tourismProject) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourismProject tourismProject, TourismProject originalTourismProject, JSONObject relationData) throws Exception {
        // tourismProject 为空的时候，无需修改主表数据。
        if (tourismProject != null && !this.update(tourismProject, originalTourismProject)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalTourismProject, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(TourismProject tourismProject, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        TourismFile tourismFile = relationData.getObject("tourismFile", TourismFile.class);
        if (tourismFile != null) {
            tourismFile.setAssociationTableId(String.valueOf(tourismProject.getId()));
            tourismFileService.saveNewOrUpdate(tourismFile,
                    tourismFileService::saveNew, tourismFileService::update);
        }
        TourismContent tourismContent = relationData.getObject("tourismContent", TourismContent.class);
        if (tourismContent != null) {
            tourismContent.setAssociationId(String.valueOf(tourismProject.getId()));
            tourismContentService.saveNewOrUpdate(tourismContent,
                    tourismContentService::saveNew, tourismContentService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(String id) {
        if (tourismProjectMapper.deleteById(id) == 0) {
            return false;
        }
        tourismFileService.removeByAssociationTableId(id);
        tourismContentService.removeByAssociationId(id);
        // 开始删除多对多父表的关联
        TourTravelNotesProjectRelation tourTravelNotesProjectRelation = new TourTravelNotesProjectRelation();
        tourTravelNotesProjectRelation.setProjectId(id);
        tourTravelNotesProjectRelationMapper.delete(new QueryWrapper<>(tourTravelNotesProjectRelation));
        return true;
    }

    @Override
    public List<TourismProject> getTourismProjectList(TourismProject filter, String orderBy) {
        return tourismProjectMapper.getTourismProjectList(filter, orderBy);
    }

    @Override
    public List<TourismProject> getTourismProjectListWithRelation(TourismProject filter, String orderBy) {
        List<TourismProject> resultList = tourismProjectMapper.getTourismProjectList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly(), batchSize);
        return resultList;
    }

    @Override
    public List<TourismProject> getNotInTourismProjectListByTravelNotesId(String travelNotesId, TourismProject filter, String orderBy) {
        List<TourismProject> resultList;
        if (travelNotesId != null) {
            resultList = tourismProjectMapper.getNotInTourismProjectListByTravelNotesId(travelNotesId, filter, orderBy);
        } else {
            resultList = getTourismProjectList(filter, orderBy);
        }
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        return resultList;
    }

    @Override
    public List<TourismProject> getTourismProjectListByTravelNotesId(String travelNotesId, TourismProject filter, String orderBy) {
        List<TourismProject> resultList =
                tourismProjectMapper.getTourismProjectListByTravelNotesId(travelNotesId, filter, orderBy);
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        // 这里是为中间表TourTravelNotesProjectRelation中关联的静态字典，进行基于注解的数据绑定。
        List<TourTravelNotesProjectRelation> tourTravelNotesProjectRelationList = resultList.stream()
                .filter(c -> c.getTourTravelNotesProjectRelation() != null)
                .map(TourismProject::getTourTravelNotesProjectRelation).collect(Collectors.toList());
        tourTravelNotesProjectRelationService.buildRelationForDataList(
                tourTravelNotesProjectRelationList, MyRelationParam.dictOnly(), CollUtil.newHashSet("travelNotesId", "projectId"));
        return resultList;
    }

    @Override
    public Map<String, List<TourismProject>> getTourismProjectListMapByTourTourismProjectTravelNotesList(
            List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList, TourismProject filter, String orderBy) {
        List<String> travelNotesIds = tourTourismProjectTravelNotesList.stream().map(TourTourismProjectTravelNotes::getId).collect(Collectors.toList());
        List<TourismProject> resultList =
                tourismProjectMapper.getTourismProjectListByTravelNotesIds(travelNotesIds, filter, orderBy);
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly());
        return resultList.stream().collect(Collectors.groupingBy(s -> s.getTourTravelNotesProjectRelation().getTravelNotesId()));
    }

    @Override
    public CallResult verifyImportList(List<TourismProject> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "isHotspot")) {
            callResult = verifyImportForConstDict(dataList, "isHotspotDictMap", TourismProject::getIsHotspot);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", TourismProject::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "belongTab")) {
            callResult = verifyImportForDatasourceDict(dataList, "belongTabDictMap", TourismProject::getBelongTab);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "id")) {
            callResult = verifyImportForOneToOneRelation(dataList, "TourismFile", TourismProject::getId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourismProject tourismProject, TourismProject originalTourismProject) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourismProject, originalTourismProject, TourismProject::getBelongTab)
                && !directoryInfoService.existId(tourismProject.getBelongTab())) {
            return CallResult.error(String.format(errorMessageFormat, "所属分类"));
        }
        return CallResult.ok();
    }

    private TourismProject buildDefaultValue(TourismProject tourismProject) {
        if (tourismProject.getId() == null) {
            tourismProject.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourismProject);
        tourismProject.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourismProject, "priceUnit", "");
        MyModelUtil.setDefaultValue(tourismProject, "serviceEnsure", "");
        MyModelUtil.setDefaultValue(tourismProject, "contactNumber", "");
        MyModelUtil.setDefaultValue(tourismProject, "sellingPoint", "");
        MyModelUtil.setDefaultValue(tourismProject, "shortTitle", "");
        MyModelUtil.setDefaultValue(tourismProject, "shortDescription", "");
        MyModelUtil.setDefaultValue(tourismProject, "homeHotPicture", "");
        MyModelUtil.setDefaultValue(tourismProject, "contactDescription", "");
        MyModelUtil.setDefaultValue(tourismProject, "travelNotesBanner", "");
        MyModelUtil.setDefaultValue(tourismProject, "role", "");
        MyModelUtil.setDefaultValue(tourismProject, "departureTime", new Date());
        MyModelUtil.setDefaultValue(tourismProject, "averageCost", "");
        return tourismProject;
    }

    @Override
    public List<DirectoryInfo> getDirectoryInfoList(DirectoryInfo filter, String orderBy) {
        return directoryInfoService.getDirectoryInfoList(filter, orderBy);
    }
}
