package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.*;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 签证基础资料管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("visaMetarialsListService")
public class VisaMetarialsListServiceImpl extends BaseService<VisaMetarialsList, Long> implements VisaMetarialsListService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private VisaMetarialsListMapper visaMetarialsListMapper;
    @Autowired
    private TourVisaService tourVisaService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<VisaMetarialsList> mapper() {
        return visaMetarialsListMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public VisaMetarialsList saveNew(VisaMetarialsList visaMetarialsList) {
        visaMetarialsListMapper.insert(this.buildDefaultValue(visaMetarialsList));
        return visaMetarialsList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<VisaMetarialsList> visaMetarialsListList) {
        if (CollUtil.isNotEmpty(visaMetarialsListList)) {
            visaMetarialsListList.forEach(this::buildDefaultValue);
            visaMetarialsListMapper.insertList(visaMetarialsListList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<VisaMetarialsList> visaMetarialsListList, int batchSize) {
        if (CollUtil.isEmpty(visaMetarialsListList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(visaMetarialsListList.size(), start + batchSize);
            List<VisaMetarialsList> subList = visaMetarialsListList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            visaMetarialsListMapper.insertList(subList);
            if (end == visaMetarialsListList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(VisaMetarialsList visaMetarialsList, VisaMetarialsList originalVisaMetarialsList) {
        MyModelUtil.fillCommonsForUpdate(visaMetarialsList, originalVisaMetarialsList);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<VisaMetarialsList> uw = this.createUpdateQueryForNullValue(visaMetarialsList, visaMetarialsList.getId());
        return visaMetarialsListMapper.update(visaMetarialsList, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return visaMetarialsListMapper.deleteById(id) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int removeByVisaId(Long visaId) {
        VisaMetarialsList deletedObject = new VisaMetarialsList();
        deletedObject.setVisaId(visaId);
        return visaMetarialsListMapper.delete(new QueryWrapper<>(deletedObject));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateBatchByVisaId(Long visaId, List<VisaMetarialsList> dataList) {
        this.updateBatchOneToManyRelation("visaId", visaId,
                null, null, dataList, this::saveNewBatch);
    }

    @Override
    public List<VisaMetarialsList> getVisaMetarialsListList(VisaMetarialsList filter, String orderBy) {
        return visaMetarialsListMapper.getVisaMetarialsListList(filter, orderBy);
    }

    @Override
    public List<VisaMetarialsList> getVisaMetarialsListListWithRelation(VisaMetarialsList filter, String orderBy) {
        List<VisaMetarialsList> resultList = visaMetarialsListMapper.getVisaMetarialsListList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<VisaMetarialsList> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(VisaMetarialsList visaMetarialsList, VisaMetarialsList originalVisaMetarialsList) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是一对多的验证
        if (this.needToVerify(visaMetarialsList, originalVisaMetarialsList, VisaMetarialsList::getVisaId)
                && !tourVisaService.existId(visaMetarialsList.getVisaId())) {
            return CallResult.error(String.format(errorMessageFormat, "签证商品id"));
        }
        return CallResult.ok();
    }

    private VisaMetarialsList buildDefaultValue(VisaMetarialsList visaMetarialsList) {
        if (visaMetarialsList.getId() == null) {
            visaMetarialsList.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(visaMetarialsList);
        visaMetarialsList.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(visaMetarialsList, "title", "");
        return visaMetarialsList;
    }
}
