package com.tourism.webadmin.back.vo;

import com.tourism.common.core.base.vo.BaseVo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 餐馆信息VO视图对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "RestaurantInfoVO视图对象")
@Data
@EqualsAndHashCode(callSuper = true)
public class RestaurantInfoVo extends BaseVo {

    /**
     * 主键id。
     */
    @Schema(description = "主键id")
    private String id;

    /**
     * 店铺名称。
     */
    @Schema(description = "店铺名称")
    private String name;

    /**
     * 店铺标签。
     */
    @Schema(description = "店铺标签")
    private String tag;

    /**
     * 店铺种类。
     */
    @Schema(description = "店铺种类")
    private String typeId;

    /**
     * 店铺公告。
     */
    @Schema(description = "店铺公告")
    private String announcement;

    /**
     * 工作时间段(例:8:00-19:00)。
     */
    @Schema(description = "工作时间段(例:8:00-19:00)")
    private String workTime;

    /**
     * 店铺地址。
     */
    @Schema(description = "店铺地址")
    private String address;

    /**
     * 是否启用，0禁用，1启用。
     */
    @Schema(description = "是否启用，0禁用，1启用")
    private Integer enable;

    /**
     * 店铺图片。
     */
    @Schema(description = "店铺图片")
    private String url;

    /**
     * 店铺环境图片。
     */
    @Schema(description = "店铺环境图片")
    private String environmentImage;

    /**
     * banner图。
     */
    @Schema(description = "banner图")
    private String restaurantBanner;

    /**
     * 推荐指数。
     */
    @Schema(description = "推荐指数")
    private Long recommendationRate;

    /**
     * 联系方式。
     */
    @Schema(description = "联系方式")
    private String contactInformation;

    /**
     * 经度。
     */
    @Schema(description = "经度")
    private BigDecimal longitude;

    /**
     * 纬度。
     */
    @Schema(description = "纬度")
    private BigDecimal latitude;

    /**
     * 销量。
     */
    @Schema(description = "销量")
    private Long totalSales;

    /**
     * 所处国家。
     */
    @Schema(description = "所处国家")
    private String contry;

    /**
     * 所处城市。
     */
    @Schema(description = "所处城市")
    private String city;

    /**
     * 用户所在部门Id。
     */
    @Schema(description = "用户所在部门Id")
    private String deptId;

    /**
     * 配送费。
     */
    @Schema(description = "配送费")
    private String deliveryCharge;

    /**
     * 最低消费。
     */
    @Schema(description = "最低消费")
    private BigDecimal minCharge;

    /**
     * 精选好店 标记 1是 2 不是。
     */
    @Schema(description = "精选好店 标记 1是 2 不是")
    private Integer isSelectShop;

    /**
     * 推荐店铺 标记 1是 2 不是。
     */
    @Schema(description = "推荐店铺 标记 1是 2 不是")
    private Integer isRecommendedShop;

    /**
     * 美味店铺标记  1是 2不是。
     */
    @Schema(description = "美味店铺标记  1是 2不是")
    private Integer isDeliciousShop;

    /**
     * 优选新店 标记 1是 2 不是。
     */
    @Schema(description = "优选新店 标记 1是 2 不是")
    private Integer isBestNewShop;

    /**
     * 送餐时间/min。
     */
    @Schema(description = "送餐时间/min")
    private Integer deliveryTime;



    /**
     * typeId 字典关联数据。
     */
    @Schema(description = "typeId 字典关联数据")
    private Map<String, Object> typeIdDictMap;

    /**
     * typeId 字典关联数据。
     */
    @Schema(description = "typeId 字典关联数据")
    private List<Map<String, Object>> typeIdDictMapList;

    /**
     * deptId 字典关联数据。
     */
    @Schema(description = "deptId 字典关联数据")
    private Map<String, Object> deptIdDictMap;

    /**
     * enable 常量字典关联数据。
     */
    @Schema(description = "enable 常量字典关联数据")
    private Map<String, Object> enableDictMap;


    /**
     * isSelectShop 常量字典关联数据。
     */
    @Schema(description = "isSelectShop 常量字典关联数据")
    private Map<String, Object> isSelectShopDictMap;

    /**
     * isRecommendedShop 常量字典关联数据。
     */
    @Schema(description = "isRecommendedShop 常量字典关联数据")
    private Map<String, Object> isRecommendedShopDictMap;

    /**
     * isDeliciousShop 常量字典关联数据。
     */
    @Schema(description = "isDeliciousShop 常量字典关联数据")
    private Map<String, Object> isDeliciousShopDictMap;

    /**
     * isBestNewShop 常量字典关联数据。
     */
    @Schema(description = "isBestNewShop 常量字典关联数据")
    private Map<String, Object> isBestNewShopDictMap;

    /**
     * urlAfterConvert url转换后的地址。
     */
    @Schema(description = "urlAfterConvert url转换后的地址")
    private List<String> urlAfterConvert;

    private List<String> typeIdList;

    private String typeName;

    /**
     * environmentImageAfterConvert environmentImage转换后的地址。
     */
    @Schema(description = "environmentImageAfterConvert url转换后的地址")
    private List<String> environmentImageAfterConvert;

    /**
     * restaurantBannerAfterConvert environmentImage转换后的地址。
     */
    @Schema(description = "restaurantBannerAfterConvert url转换后的地址")
    private List<String> restaurantBannerAfterConvert;

    private BannerInfoVo bannerInfoVo;
}
