package com.tourism.webadmin.back.vo;

import com.baomidou.mybatisplus.annotation.TableField;
import com.tourism.common.core.annotation.RelationConstDict;
import com.tourism.common.core.base.vo.BaseVo;
import com.tourism.webadmin.back.model.constant.ProjectType;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * 门户预定管理VO视图对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "TourBookInfoVO视图对象")
@Data
@EqualsAndHashCode(callSuper = true)
public class TourBookInfoVo extends BaseVo {

    /**
     * 主键id。
     */
    @Schema(description = "主键id")
    private Long id;

    /**
     * 预定电话。
     */
    @Schema(description = "预定电话")
    private String bookMobile;

    /**
     * 预定姓名。
     */
    @Schema(description = "预定姓名")
    private String bookName;

    /**
     * 预定的ip。
     */
    @Schema(description = "预定的ip")
    private String bookIp;

    /**
     * 预定时间。
     */
    @Schema(description = "预定时间")
    private Date bookTime;

    /**
     * 是否处理(0:未处理 1:已处理)。
     */
    @Schema(description = "是否处理(0:未处理 1:已处理)")
    private Integer isHandle;

    /**
     * 预定的种类(1:旅游 2:劳务)。
     */
    @Schema(description = "预定的种类(1:旅游 2:劳务)")
    private Integer type;

    /**
     * 预定种类的id。
     */
    @Schema(description = "预定种类的id")
    private String projectId;

    /**
     * 成人价格。
     */
    @Schema(description = "成人价格")
    private BigDecimal adultPrice;

    /**
     * 儿童价格。
     */
    @Schema(description = "儿童价格")
    private BigDecimal childrenPrice;

    /**
     * 成人人数。
     */
    @Schema(description = "成人人数")
    private Integer adultNumber;

    /**
     * 儿童人数。
     */
    @Schema(description = "儿童人数")
    private Integer childrenNumber;

    /**
     * 总价格。
     */
    @Schema(description = "总价格")
    private BigDecimal totalPrice;

    /**
     * projectId 的一对一关联数据对象，数据对应类型为TourismProjectVo。
     */
    @Schema(description = "projectId 的一对一关联数据对象，数据对应类型为TourismProjectVo")
    private Map<String, Object> tourismProject;

    /**
     * type 常量字典关联数据。
     */
    @Schema(description = "type 常量字典关联数据")
    private Map<String, Object> typeDictMap;

    /**
     * isHandle 全局字典关联数据。
     */
    @Schema(description = "isHandle 全局字典关联数据")
    private Map<String, Object> isHandleDictMap;

    /**
     * projectMap 项目关联的数据。
     */
    @Schema(description = "projectMap 项目关联的数据")
    private JobProjectVo jobProjectVo;
}
