package com.tourism.webadmin.back.vo;

import com.tourism.common.core.base.vo.BaseVo;
import com.tourism.webadmin.back.model.TourHouseEquipment;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * 租赁房屋信息VO视图对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "租赁房屋信息VO视图对象")
@Data
@EqualsAndHashCode(callSuper = true)
public class TourHouseRentInfoVo extends BaseVo {

    /**
     * 主键。
     */
    @Schema(description = "主键")
    private String id;

    /**
     * 所属地区类型。
     */
    @Schema(description = "所属地区类型")
    private Long areaId;

    /**
     * 顾问id。
     */
    @Schema(description = "顾问id")
    private Long advisorId;

    /**
     * 租赁方式 0整租 1合租 2个人房源 3短租。
     */
    @Schema(description = "租赁方式 0整租 1合租 2个人房源 3短租")
    private Integer rentType;

    /**
     * 厅室类型。
     */
    @Schema(description = "厅室类型")
    private Integer officeType;

    /**
     * 租金。
     */
    @Schema(description = "租金")
    private BigDecimal rentPrice;

    /**
     * 面积。
     */
    @Schema(description = "面积")
    private Integer area;

    /**
     * 房屋类型 别墅、公寓...。
     */
    @Schema(description = "房屋类型 别墅、公寓...")
    private Integer houseType;

    /**
     * 租金类型(0无租金;1押一付一;2押一付二;3押一付三....)。
     */
    @Schema(description = "租金类型(0无租金;1押一付一;2押一付二;3押一付三....)")
    private Integer rentKind;

    /**
     * 朝向类型。
     */
    @Schema(description = "朝向类型")
    private Integer orientation;

    /**
     * 所在楼层。
     */
    @Schema(description = "所在楼层")
    private Integer currentFloor;

    /**
     * 装修类型。
     */
    @Schema(description = "装修类型")
    private Integer decoration;

    /**
     * 楼盘名称/所属小区。
     */
    @Schema(description = "楼盘名称/所属小区")
    private String communityName;

    /**
     * 地址。
     */
    @Schema(description = "地址")
    private String address;

    /**
     * 货币单位。
     */
    @Schema(description = "货币单位")
    private String currency;

    /**
     * 所属区域。
     */
    @Schema(description = "所属区域")
    private String region;

    /**
     * 首页封面图。
     */
    @Schema(description = "首页封面图")
    private String coverUrl;

    /**
     * 房屋标题。
     */
    @Schema(description = "房屋标题")
    private String houseTitle;

    /**
     * 总楼层。
     */
    @Schema(description = "总楼层")
    private Integer totalFloor;

    /**
     * 二维码。
     */
    @Schema(description = "二维码")
    private String code;

    /**
     * 联系电话。
     */
    @Schema(description = "联系电话")
    private String mobile;

    /**
     * 备注。
     */
    @Schema(description = "备注")
    private String remark;

    /**
     * 排序顺序。
     */
    @Schema(description = "排序顺序")
    private Integer showOrder;

    /**
     * 租房状态(0:未租出;1:已租出)。
     */
    @Schema(description = "租房状态(0:未租出;1:已租出)")
    private Integer rentState;

    /**
     * 楼层类型(1:低楼层;2:中楼层;3:高楼层)。
     */
    @Schema(description = "楼层类型(1:低楼层;2:中楼层;3:高楼层)")
    private Integer floorType;

    /**
     * id 的一对一关联数据对象，数据对应类型为TourHouseEquipmentVo。
     */
    @Schema(description = "id 的一对一关联数据对象，数据对应类型为TourHouseEquipmentVo")
    private TourHouseEquipmentVo tourHouseEquipment;

    /**
     * areaId 字典关联数据。
     */
    @Schema(description = "areaId 字典关联数据")
    private Map<String, Object> areaIdDictMap;

    /**
     * rentState 常量字典关联数据。
     */
    @Schema(description = "rentState 常量字典关联数据")
    private Map<String, Object> rentStateDictMap;

    /**
     * rentType 全局字典关联数据。
     */
    @Schema(description = "rentType 全局字典关联数据")
    private Map<String, Object> rentTypeDictMap;

    /**
     * officeType 全局字典关联数据。
     */
    @Schema(description = "officeType 全局字典关联数据")
    private Map<String, Object> officeTypeDictMap;

    /**
     * houseType 全局字典关联数据。
     */
    @Schema(description = "houseType 全局字典关联数据")
    private Map<String, Object> houseTypeDictMap;

    /**
     * rentKind 全局字典关联数据。
     */
    @Schema(description = "rentKind 全局字典关联数据")
    private Map<String, Object> rentKindDictMap;

    /**
     * orientation 全局字典关联数据。
     */
    @Schema(description = "orientation 全局字典关联数据")
    private Map<String, Object> orientationDictMap;

    /**
     * decoration 全局字典关联数据。
     */
    @Schema(description = "decoration 全局字典关联数据")
    private Map<String, Object> decorationDictMap;

    /**
     * advisorId 字典关联数据。
     */
    @Schema(description = "advisorId 字典关联数据")
    private Map<String, Object> advisorIdDictMap;

    /**
     * advisorId 的一对一关联数据对象，数据对应类型为TourHouseAdvisorInfoVo。
     */
    @Schema(description = "advisorId 的一对一关联数据对象，数据对应类型为TourHouseAdvisorInfoVo")
    private TourHouseAdvisorInfoVo tourHouseAdvisorInfo;

    /**
     * floorType 全局字典关联数据。
     */
    @Schema(description = "floorType 全局字典关联数据")
    private Map<String, Object> floorTypeDictMap;

    /**
     * 是否热推房源。
     */
    @Schema(description = "是否热推房源")
    private Integer isHotPush;

    /**
     * isHotPush 常量字典关联数据。
     */
    @Schema(description = "isHotPush 常量字典关联数据")
    private Map<String, Object> isHotPushDictMap;
}
